@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -29,10 +29,11 @@ import datetime
 from decimal import Decimal
 
 from .constants import (
-    FieldFlag, ServerCmd, FieldType, ClientFlag)
+    FieldFlag, ServerCmd, FieldType, ClientFlag, MAX_MYSQL_TABLE_COLUMNS)
 from . import errors, utils
 from .authentication import get_auth_plugin
 from .catch23 import PY2, struct_unpack
+from .errors import get_exception
 
 
 class MySQLProtocol(object):
@@ -173,6 +174,7 @@ class MySQLProtocol(object):
          capabilities2,
          auth_data_length
         ) = struct_unpack('<I8sx2sBH2sBxxxxxxxxxx', packet[0:31])
+        res['server_version_original'] = res['server_version_original'].decode()
 
         packet = packet[31:]
 
@@ -186,8 +188,13 @@ class MySQLProtocol(object):
                 auth_data2 = auth_data2[:-1]
 
         if capabilities & ClientFlag.PLUGIN_AUTH:
-            (packet, res['auth_plugin']) = utils.read_string(
-                packet, end=b'\x00')
+            if (b'\x00' not in packet
+                    and res['server_version_original'].startswith("5.5.8")):
+                # MySQL server 5.5.8 has a bug where end byte is not send
+                (packet, res['auth_plugin']) = (b'', packet)
+            else:
+                (packet, res['auth_plugin']) = utils.read_string(
+                    packet, end=b'\x00')
             res['auth_plugin'] = res['auth_plugin'].decode('utf-8')
         else:
             res['auth_plugin'] = 'mysql_native_password'
@@ -199,14 +206,14 @@ class MySQLProtocol(object):
     def parse_ok(self, packet):
         """Parse a MySQL OK-packet"""
         if not packet[4] == 0:
-            raise errors.InterfaceError("Failed parsing OK packet.")
+            raise errors.InterfaceError("Failed parsing OK packet (invalid).")
 
         ok_packet = {}
         try:
             ok_packet['field_count'] = struct_unpack('<xxxxB', packet[0:5])[0]
             (packet, ok_packet['affected_rows']) = utils.read_lc_int(packet[5:])
             (packet, ok_packet['insert_id']) = utils.read_lc_int(packet)
-            (ok_packet['server_status'],
+            (ok_packet['status_flag'],
              ok_packet['warning_count']) = struct_unpack('<HH', packet[0:4])
             packet = packet[4:]
             if packet:
@@ -219,11 +226,12 @@ class MySQLProtocol(object):
     def parse_column_count(self, packet):
         """Parse a MySQL packet with the number of columns in result set"""
         try:
-            return utils.read_lc_int(packet[4:])[1]
+            count = utils.read_lc_int(packet[4:])[1]
+            return count
         except (struct.error, ValueError):
             raise errors.InterfaceError("Failed parsing column count")
 
-    def parse_column(self, packet):
+    def parse_column(self, packet, charset='utf-8'):
         """Parse a MySQL column-packet"""
         (packet, _) = utils.read_lc_string(packet[4:])  # catalog
         (packet, _) = utils.read_lc_string(packet)  # db
@@ -239,7 +247,7 @@ class MySQLProtocol(object):
             raise errors.InterfaceError("Failed parsing column information")
 
         return (
-            name.decode('utf-8'),
+            name.decode(charset),
             field_type,
             None,  # display_size
             None,  # internal_size
@@ -251,6 +259,10 @@ class MySQLProtocol(object):
 
     def parse_eof(self, packet):
         """Parse a MySQL EOF-packet"""
+        if packet[4] == 0:
+            # EOF packet deprecation
+            return self.parse_ok(packet)
+
         err_msg = "Failed parsing EOF packet."
         res = {}
         try:
@@ -265,12 +277,15 @@ class MySQLProtocol(object):
         res['status_flag'] = unpacked[3]
         return res
 
-    def parse_statistics(self, packet):
+    def parse_statistics(self, packet, with_header=True):
         """Parse the statistics packet"""
         errmsg = "Failed getting COM_STATISTICS information"
         res = {}
         # Information is separated by 2 spaces
-        pairs = packet[4:].split(b'\x20\x20')
+        if with_header:
+            pairs = packet[4:].split(b'\x20\x20')
+        else:
+            pairs = packet.split(b'\x20\x20')
         for pair in pairs:
             try:
                 (lbl, val) = [v.strip() for v in pair.split(b':', 2)]
@@ -289,7 +304,7 @@ class MySQLProtocol(object):
                         "{0} ({1}:{2}).".format(errmsg, lbl, val))
         return res
 
-    def read_text_result(self, sock, count=1):
+    def read_text_result(self, sock, version, count=1):
         """Read MySQL text result
 
         Reads all or given number of rows from the socket.
@@ -302,9 +317,7 @@ class MySQLProtocol(object):
         rowdata = None
         i = 0
         while True:
-            if eof is not None:
-                break
-            if i == count:
+            if eof or i == count:
                 break
             packet = sock.recv()
             if packet.startswith(b'\xff\xff\xff'):
@@ -313,12 +326,9 @@ class MySQLProtocol(object):
                 while packet.startswith(b'\xff\xff\xff'):
                     datas.append(packet[4:])
                     packet = sock.recv()
-                if packet[4] == 254:
-                    eof = self.parse_eof(packet)
-                else:
-                    datas.append(packet[4:])
-                rowdata = utils.read_lc_string_list(b''.join(datas))
-            elif packet[4] == 254:
+                datas.append(packet[4:])
+                rowdata = utils.read_lc_string_list(bytearray(b'').join(datas))
+            elif (packet[4] == 254 and packet[0] < 7):
                 eof = self.parse_eof(packet)
                 rowdata = None
             else:
@@ -326,8 +336,10 @@ class MySQLProtocol(object):
                 rowdata = utils.read_lc_string_list(packet[4:])
             if eof is None and rowdata is not None:
                 rows.append(rowdata)
+            elif eof is None and rowdata is None:
+                raise get_exception(packet)
             i += 1
-        return (rows, eof)
+        return rows, eof
 
     def _parse_binary_integer(self, packet, field):
         """Parse an integer from a binary packet"""
@@ -457,6 +469,8 @@ class MySQLProtocol(object):
                 values = self._parse_binary_values(columns, packet[5:])
             if eof is None and values is not None:
                 rows.append(values)
+            elif eof is None and values is None:
+                raise get_exception(packet)
             i += 1
         return (rows, eof)
 
@@ -636,24 +650,25 @@ class MySQLProtocol(object):
                     values.append(packed)
                 elif isinstance(value, str):
                     if PY2:
-                        values.append(utils.intstore(len(value)) + value)
+                        values.append(utils.lc_int(len(value)) +
+                                      value)
                     else:
                         value = value.encode(charset)
                         values.append(
-                            utils.intstore(len(value)) + value)
+                            utils.lc_int(len(value)) + value)
                     field_type = FieldType.VARCHAR
                 elif isinstance(value, bytes):
-                    values.append(utils.intstore(len(value)) + value)
+                    values.append(utils.lc_int(len(value)) + value)
                     field_type = FieldType.BLOB
                 elif PY2 and \
                         isinstance(value, unicode):  # pylint: disable=E0602
                     value = value.encode(charset)
-                    values.append(utils.intstore(len(value)) + value)
+                    values.append(utils.lc_int(len(value)) + value)
                     field_type = FieldType.VARCHAR
                 elif isinstance(value, Decimal):
                     values.append(
-                        utils.intstore(len(str(value).encode(charset))) +
-                        str(value).encode(charset))
+                        utils.lc_int(len(str(value).encode(
+                            charset))) + str(value).encode(charset))
                     field_type = FieldType.DECIMAL
                 elif isinstance(value, float):
                     values.append(struct.pack('d', value))
@@ -696,7 +711,7 @@ class MySQLProtocol(object):
                 "Failed parsing AuthSwitchRequest packet")
 
         (packet, plugin_name) = utils.read_string(packet[5:], end=b'\x00')
-        if packet[-1] == 0:
+        if packet and packet[-1] == 0:
             packet = packet[:-1]
 
         return plugin_name.decode('utf8'), packet
