@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,12 +18,12 @@
 """mysql.utilities"""
 
 # Major, Minor, Patch, Status
-VERSION = (1, 5, 6, 'GA', 0)
+VERSION = (1, 6, 5, 'GA', 0)
 # Future versions will have to include only the X, Y (no Z).
 
 VERSION_STRING = "%s.%s.%s" % VERSION[0:3]
 
-COPYRIGHT = "2010, 2015 Oracle and/or its affiliates. All rights reserved."
+COPYRIGHT = "2010, 2017 Oracle and/or its affiliates. All rights reserved."
 
 COPYRIGHT_FULL = "Copyright (c) " + COPYRIGHT + """
 This is a release of dual licensed MySQL Utilities. For the avoidance of
@@ -48,6 +48,9 @@ CONNECTOR_MIN_VERSION = (1, 2, 1)
 AVAILABLE_UTILITIES = {
     'mysqlauditadmin': (),
     'mysqlauditgrep': (),
+    'mysqlbinlogmove': (),
+    'mysqlbinlogpurge': (),
+    'mysqlbinlogrotate': (),
     'mysqldbcompare': (),
     'mysqldbcopy': (),
     'mysqldbexport': (),
@@ -56,6 +59,7 @@ AVAILABLE_UTILITIES = {
     'mysqldiskusage': (),
     'mysqlfailover': (),
     'mysqlfrm': (),
+    'mysqlgrants': (),
     'mysqlindexcheck': (),
     'mysqlmetagrep': (),
     'mysqlprocgrep': (),
@@ -67,6 +71,7 @@ AVAILABLE_UTILITIES = {
     'mysqlrplsync': (),
     'mysqlserverclone': (),
     'mysqlserverinfo': (),
+    'mysqlslavetrx': (),
     'mysqluc': (),
     'mysqluserclone': (),
 }
