@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2014 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -137,7 +137,7 @@ def _get_db_dir_size(folder):
         return None
     for item in os.listdir(folder):
         name, ext = os.path.splitext(item)
-        if ext.upper() not in (".MYD", ".MYI") and \
+        if ext.upper() not in (".MYD", ".MYI", ".IBD") and \
            name.upper() not in ('SLOW_LOG', 'GENERAL_LOG'):
             itemfolder = os.path.join(folder, item)
             if os.path.isfile(itemfolder):
@@ -282,7 +282,7 @@ def _build_innodb_list(per_table, folder, datadir, specs, verbosity=0):
     files. It will search all database directories if per_table is True.
     Returns total size of all files found.
 
-    The verbosity option controls how much data is shown:
+    The verbosity argument controls how much data is shown:
           0 : no additional information
         > 0 : include type and specification (for shared tablespaces)
 
@@ -297,6 +297,7 @@ def _build_innodb_list(per_table, folder, datadir, specs, verbosity=0):
     total_size = 0
     tablespaces = []
     # Here, we want to capture log files as well as tablespace files.
+    # pylint: disable=R0101
     if specs is not None:
         for item in os.listdir(folder):
             name, _ = os.path.splitext(item)
@@ -357,7 +358,7 @@ def _build_db_list(server, rows, include_list, datadir, fmt=False,
     constant width using locale-specific options for printing numerals. For
     example, US locale formats 12345 as 12,345.
 
-    The verbosity option controls how much data is shown:
+    The verbosity argument controls how much data is shown:
            0 : no additional information
          > 0 : include data size (calculated) and size of misc files
         >= 2 : also include database directory actual size
@@ -377,7 +378,6 @@ def _build_db_list(server, rows, include_list, datadir, fmt=False,
 
     total = 0
     results = []
-    max_col = 0
 
     # build the list
     for row in rows:
@@ -397,10 +397,10 @@ def _build_db_list(server, rows, include_list, datadir, fmt=False,
             db_total = 0
         else:
             data_size = int(row[1])
-            db_total = int(row[1]) + misc_files
+            db_total = dbdir_size
 
         # Count total for all databases
-        total += data_size + misc_files
+        total += dbdir_size
 
         if have_read and not is_remote:
             if verbosity >= 2:  # get all columns
@@ -461,6 +461,7 @@ def _build_db_list(server, rows, include_list, datadir, fmt=False,
     else:
         fmt_rows = results
 
+    # pylint: disable=R0101
     if include_empty:
         dbs = server.exec_query("SHOW DATABASES")
         if len(fmt_rows) != len(dbs) - 1:
@@ -476,7 +477,7 @@ def _build_db_list(server, rows, include_list, datadir, fmt=False,
                     if fmt:
                         fmt_data = ['', '', '', '', '']
                         for i in range(0, num_cols):
-                            if type(row[i + 1]) == type(int):
+                            if isinstance(row[i + 1], int):
                                 fmt_data[i] = locale.format("%s",
                                                             int(row[i + 1]),
                                                             grouping=True)
@@ -706,8 +707,8 @@ def show_log_usage(server, datadir, options):
         else:
             # Requires SUPER for server < 5.6.6 or also REPLICATION CLIENT for
             # server >= 5.6.6 to execute: SHOW BINARY LOGS.
-            if (server.check_version_compat(5, 6, 6)
-               and not has_super and not has_rpl_client):
+            if (server.check_version_compat(5, 6, 6) and
+                    not has_super and not has_rpl_client):
                 print("# {0} information not accessible. User must have the "
                       "SUPER or REPLICATION CLIENT "
                       "privilege.".format(log_type.capitalize()))
@@ -838,6 +839,7 @@ def show_innodb_usage(server, datadir, options):
 
     # Check to see if innodb_file_per_table is ON
     res = server.show_server_variable('innodb_file_per_table')
+    # pylint: disable=R0102
     if res != [] and res[0][1].upper() == "ON":
         innodb_file_per_table = True
     else:
@@ -907,10 +909,10 @@ def show_innodb_usage(server, datadir, options):
                 if tablespace[1] != 'log file':
                     parts = tablespace[3].split(":")
                     if len(parts) > 2:
-                        size = int(tablespace[1]) / _MB
+                        ts_size = int(tablespace[1]) / _MB
                         print "Tablespace %s can be " % tablespace[3] + \
                               "extended by using %s:%sM[...]\n" % \
-                              (parts[0], size)
+                              (parts[0], ts_size)
     elif is_remote:
         print("# InnoDB data information not accessible from a remote host.")
     else:
@@ -923,9 +925,9 @@ def show_innodb_usage(server, datadir, options):
             if len(res) > 1:
                 raise UtilError("Found multiple rows for freespace.")
             else:
-                size = int(res[0][0])
+                fs_size = int(res[0][0])
                 if not quiet:
-                    _print_size("InnoDB freespace = ", size)
+                    _print_size("InnoDB freespace = ", fs_size)
                     print
 
     return True
