@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 #
 
 """
-This file contains the replication administration commands to manage a
+This file contains the replication administration tools for managine a
 simple master-to-slaves topology.
 """
 
@@ -28,8 +28,10 @@ import time
 from datetime import datetime, timedelta
 
 from mysql.utilities.exception import UtilRplError
+from mysql.utilities.common.gtid import gtid_set_itemize
 from mysql.utilities.common.ip_parser import hostname_is_ip
 from mysql.utilities.common.messages import (ERROR_SAME_MASTER,
+                                             ERROR_USER_WITHOUT_PRIVILEGES,
                                              HOST_IP_WARNING,
                                              EXTERNAL_SCRIPT_DOES_NOT_EXIST,
                                              INSUFFICIENT_FILE_PERMISSIONS)
@@ -38,6 +40,7 @@ from mysql.utilities.common.format import print_list
 from mysql.utilities.common.topology import Topology
 from mysql.utilities.command.failover_console import FailoverConsole
 from mysql.utilities.command.failover_daemon import FailoverDaemon
+from mysql.utilities.common.server import check_hostname_alias, get_port
 
 _VALID_COMMANDS_TEXT = """
 Available Commands:
@@ -142,6 +145,102 @@ def purge_log(filename, age):
     return True
 
 
+def skip_slaves_trx(gtid_set, slaves_cnx_val, options):
+    """Skip transactions on slaves.
+
+    This method skips the given transactions (GTID set) on all the specified
+    slaves. That is, an empty transaction is injected for each GTID in
+    the given set for one of each slaves. In case a slave already has an
+    executed transaction for a given GTID then that GTID is ignored for this
+    slave.
+
+    gtid_set[in]            String representing the set of GTIDs to skip.
+    slaves_cnx_val[in]      List of the dictionaries with the connection
+                            values for each target slave.
+    options[in]             Dictionary of options (dry_run, verbosity).
+
+    Throws an UtilError exception if an error occurs during the execution.
+    """
+    verbosity = options.get('verbosity')
+    dryrun = options.get('dry_run')
+
+    # Connect to slaves.
+    rpl_topology = Topology(None, slaves_cnx_val, options)
+
+    # Check required privileges.
+    errors = rpl_topology.check_privileges(skip_master=True)
+    if errors:
+        err_details = ''
+        for err in errors:
+            err_msg = ERROR_USER_WITHOUT_PRIVILEGES.format(
+                user=err[0], host=err[1], port=err[2],
+                operation='inject empty transactions', req_privileges=err[3])
+            err_details = '{0}{1}\n'.format(err_details, err_msg)
+        err_details.strip()
+        raise UtilRplError("Not enough privileges.\n{0}".format(err_details))
+
+    # GTID must be enabled on all servers.
+    srv_list = rpl_topology.get_servers_with_gtid_not_on()
+    if srv_list:
+        if verbosity:
+            print("# Slaves with GTID not enabled:")
+            for srv in srv_list:
+                msg = "#  - GTID_MODE={0} on {1}:{2}".format(srv[2], srv[0],
+                                                             srv[1])
+                print(msg)
+        raise UtilRplError(_GTID_ON_REQ.format(action='Transaction skip'))
+
+    if dryrun:
+        print("#")
+        print("# WARNING: Executing utility in dry run mode (read only).")
+
+    # Get GTID set that can be skipped, i.e., not in GTID_EXECUTED.
+    gtids_by_slave = rpl_topology.slaves_gtid_subtract_executed(gtid_set)
+
+    # Output GTID set that will be skipped.
+    print("#")
+    print("# GTID set to be skipped for each server:")
+    has_gtid_to_skip = False
+    for host, port, gtids_to_skip in gtids_by_slave:
+        if not gtids_to_skip:
+            gtids_to_skip = 'None'
+        else:
+            # Set flag to indicate that there is at least one GTID to skip.
+            has_gtid_to_skip = True
+        print("# - {0}@{1}: {2}".format(host, port, gtids_to_skip))
+
+    # Create dictionary to directly access the slaves instances.
+    slaves_dict = rpl_topology.get_slaves_dict()
+
+    # Skip transactions for the given list of slaves.
+    print("#")
+    # pylint: disable=R0101
+    if has_gtid_to_skip:
+        for host, port, gtids_to_skip in gtids_by_slave:
+            if gtids_to_skip:
+                # Decompose GTID set into a list of single transactions.
+                gtid_items = gtid_set_itemize(gtids_to_skip)
+                dryrun_mark = '(dry run) ' if dryrun else ''
+                print("# {0}Injecting empty transactions for '{1}:{2}'"
+                      "...".format(dryrun_mark, host, port))
+                slave_key = '{0}@{1}'.format(host, port)
+                slave_srv = slaves_dict[slave_key]['instance']
+                for uuid, trx_list in gtid_items:
+                    for trx_num in trx_list:
+                        trx_to_skip = '{0}:{1}'.format(uuid, trx_num)
+                        if verbosity:
+                            print("# - {0}".format(trx_to_skip))
+                        if not dryrun:
+                            # Inject empty transaction.
+                            slave_srv.inject_empty_trx(
+                                trx_to_skip, gtid_next_automatic=False)
+                if not dryrun:
+                    slave_srv.set_gtid_next_automatic()
+    else:
+        print("# No transaction to skip.")
+    print("#\n#...done.\n#")
+
+
 class RplCommands(object):
     """Replication commands.
 
@@ -325,7 +424,7 @@ class RplCommands(object):
                 if host_port:
                     host = host_port[0]
                 if (not host or uses_ip != hostname_is_ip(slave.host) or
-                   uses_ip != hostname_is_ip(host)):
+                        uses_ip != hostname_is_ip(host)):
                     return False
         return True
 
@@ -339,13 +438,13 @@ class RplCommands(object):
         """
         # Check new master is not actual master - need valid candidate
         candidate = self.options.get("new_master", None)
-        if (self.topology.master.is_alias(candidate['host']) and
-           self.master_vals['port'] == candidate['port']):
-            err_msg = ERROR_SAME_MASTER.format(candidate['host'],
-                                               candidate['port'],
-                                               self.master_vals['host'],
-                                               self.master_vals['port'])
-            self._report(err_msg, logging.WARN)
+        if check_hostname_alias(self.master_vals, candidate):
+            err_msg = ERROR_SAME_MASTER.format(
+                n_master_host=candidate['host'],
+                n_master_port=candidate['port'],
+                master_host=self.master_vals['host'],
+                master_port=self.master_vals['port']
+            )
             self._report(err_msg, logging.CRITICAL)
             raise UtilRplError(err_msg)
 
@@ -389,11 +488,20 @@ class RplCommands(object):
             print("# WARNING: {0}".format(warn_msg))
             self._report(warn_msg, logging.WARN, False)
 
-        self._report(" ".join(["# Performing switchover from master at",
-                     "%s:%s" % (self.master_vals['host'],
-                                self.master_vals['port']),
-                               "to slave at %s:%s." %
-                               (candidate['host'], candidate['port'])]))
+        # Fix ports before continuing.
+        candidate_port = candidate['port']
+        if os.name == "posix":
+            if self.topology.master.socket:
+                self.master_vals['port'] = self.topology.master.port
+            # Requires quick connection to server to get correct port
+            port = get_port(candidate)
+            if port:
+                candidate_port = port
+
+        self._report(" ".join(
+            ["# Performing switchover from master at",
+             "%s:%s" % (self.master_vals['host'], self.master_vals['port']),
+             "to slave at %s:%s." % (candidate['host'], candidate_port)]))
         if not self.topology.switchover(candidate):
             self._report("# Errors found. Switchover aborted.", logging.ERROR)
             return False
@@ -426,12 +534,13 @@ class RplCommands(object):
                          "then slaves list.")
         best_slave = self.topology.find_best_slave(candidates)
         if best_slave is None:
-            self._report("ERROR: No slave found that meets eligilibility "
+            self._report("ERROR: No slave found that meets eligibility "
                          "requirements.", logging.ERROR)
             return
 
-        self._report("# Best slave found is located on %s:%s." %
-                     (best_slave['host'], best_slave['port']))
+        # Get the correct port
+        self._report("# Best slave found is located on {0}:{1}."
+                     "".format(best_slave['host'], best_slave['port']))
 
     def _failover(self, strict=False, options=None):
         """Perform failover
@@ -714,6 +823,7 @@ class RplCommands(object):
         exec_fail = self.options.get("exec_fail", None)
         post_fail = self.options.get("post_fail", None)
         pedantic = self.options.get('pedantic', False)
+        fail_retry = self.options.get('fail_retry', None)
 
         # Only works for GTID_MODE=ON
         if not self.topology.gtid_enabled():
@@ -787,9 +897,10 @@ class RplCommands(object):
                 else:
                     self._report("# Spawning external script for failover "
                                  "checking.")
-                    res = execute_script(exec_fail, None,
-                                         [old_host, old_port], self.verbose)
-                    if res == 0:
+                    script_res = execute_script(exec_fail, None,
+                                                [old_host, old_port],
+                                                self.verbose)
+                    if script_res == 0:
                         self._report("# Failover check script completed Ok. "
                                      "Failover averted.")
                     else:
@@ -810,6 +921,19 @@ class RplCommands(object):
                     except:
                         pass
 
+                # If user specified a master fail retry, wait for the
+                # predetermined time and attempt to check the master again.
+                if fail_retry is not None and \
+                   not self.topology.master.is_alive():
+                    msg = "Master is still not reachable. Waiting for %s " \
+                          "seconds to retry detection." % fail_retry
+                    self._report(msg, logging.INFO, False)
+                    time.sleep(fail_retry)
+                    try:
+                        self.topology.master.connect()
+                    except:
+                        pass
+
                 # Check the master again. If no connection or lost connection,
                 # try ping. This performs the timeout threshold for detecting
                 # a down master. If still not alive, try to reconnect and if
@@ -832,6 +956,9 @@ class RplCommands(object):
                     if failover:
                         self._report("Failed to reconnect to the master after "
                                      "3 attemps.", logging.INFO)
+                    else:
+                        self._report("Master is Ok. Resuming watch.",
+                                     logging.INFO)
 
             if failover:
                 self._report("Master is confirmed to be down or unreachable.",
@@ -852,16 +979,24 @@ class RplCommands(object):
                                              "failover is not enabled. ")
                     self._report(msg, logging.CRITICAL, False)
                     # Execute post failover script
-                    self.topology.run_script(post_fail, False,
-                                             [old_host, old_port])
+                    try:
+                        self.topology.run_script(post_fail, False,
+                                                 [old_host, old_port])
+                    except Exception as err:  # pylint: disable=W0703
+                        self._report("# Post fail script failed! {0}"
+                                     "".format(err), level=logging.ERROR)
                     raise UtilRplError(msg, _FAILOVER_ERRNO)
                 if not res:
                     msg = _FAILOVER_ERROR % ("An error was encountered "
                                              "during failover. ")
                     self._report(msg, logging.CRITICAL, False)
                     # Execute post failover script
-                    self.topology.run_script(post_fail, False,
-                                             [old_host, old_port])
+                    try:
+                        self.topology.run_script(post_fail, False,
+                                                 [old_host, old_port])
+                    except Exception as err:  # pylint: disable=W0703
+                        self._report("# Post fail script failed! {0}"
+                                     "".format(err), level=logging.ERROR)
                     raise UtilRplError(msg)
                 self.master = self.topology.master
                 console.master = self.master
@@ -873,9 +1008,14 @@ class RplCommands(object):
                 console.clear()
                 failover = False
                 # Execute post failover script
-                self.topology.run_script(post_fail, False,
-                                         [old_host, old_port,
-                                          self.master.host, self.master.port])
+                try:
+                    self.topology.run_script(post_fail, False,
+                                             [old_host, old_port,
+                                              self.master.host,
+                                              self.master.port])
+                except Exception as err:  # pylint: disable=W0703
+                    self._report("# Post fail script failed! {0}"
+                                 "".format(err), level=logging.ERROR)
 
                 # Unregister existing instances from slaves
                 self._report("Unregistering existing instances from slaves.",
@@ -888,8 +1028,8 @@ class RplCommands(object):
                 failover_mode = console.register_instance()
 
             # discover slaves if option was specified at startup
-            elif (self.options.get("discover", None) is not None
-                  and not first_pass):
+            elif (self.options.get("discover", None) is not None and
+                  not first_pass):
                 # Force refresh of health list if new slaves found
                 if self.topology.discover_slaves():
                     console.list_data = None
