@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -67,8 +67,9 @@ _COLUMNS.extend(_SERVER_VARIABLES)
 
 # Retrieve column names from the _LOG_FILES_VARIABLES, filter the
 # None value, sort them alphabetically and add them to the  _COLUMNS list
-_COLUMNS.extend(sorted(
-    val for val in chain(*_LOG_FILES_VARIABLES.values()) if val is not None)
+_COLUMNS.extend(
+    sorted(val for val in chain(
+        *_LOG_FILES_VARIABLES.values()) if val is not None)
 )
 
 # Used to get O(1) performance in checking if an item is already present
@@ -148,7 +149,7 @@ def _server_info(server_val, get_defaults=False, options=None):
     params_dict['warnings'] = []
 
     # Identify server by string: 'host:port[:socket]'.
-    server_id = "{0}:{1}".format(source_values['host'], source_values['port'])
+    server_id = "{0}:{1}".format(server.host, server.port)
     if source_values.get('socket', None):
         server_id = "{0}:{1}".format(server_id, source_values.get('socket'))
     params_dict['server'] = server_id
@@ -162,6 +163,9 @@ def _server_info(server_val, get_defaults=False, options=None):
             raise UtilError("Unable to determine {0} of server '{1}'"
                             ".".format(server_var, server_id))
 
+    # Verify if the server is a local server.
+    server_is_local = server.is_alias('localhost')
+
     # Get _LOG_FILES_VARIABLES values from the server
     for msg, log_tpl in _LOG_FILES_VARIABLES.iteritems():
         res = server.show_server_variable(log_tpl.log_name)
@@ -181,9 +185,22 @@ def _server_info(server_val, get_defaults=False, options=None):
             else:  # log error, so log_file_name is already on params_dict
                 log_file = params_dict[log_tpl.log_name]
 
-            # Now get the information about the size of the logs
+            # Size can only be obtained from the files of a local server.
+            if not server_is_local:
+                params_dict[log_tpl.log_file_size] = 'UNAVAILABLE'
+                # Show warning about log size unaviable.
+                params_dict['warnings'].append("Unable to get information "
+                                               "regarding variable '{0}' "
+                                               "from a remote server."
+                                               "".format(msg))
             # If log file is stderr, we cannot get the correct size.
-            if log_file not in ["stderr", "stdout"]:
+            elif log_file in ["stderr", "stdout"]:
+                params_dict[log_tpl.log_file_size] = 'UNKNOWN'
+                # Show warning about log unknown size.
+                params_dict['warnings'].append("Unable to get size information"
+                                               " from '{0}' for '{1}'."
+                                               "".format(log_file, msg))
+            else:
                 # Now get the information about the size of the logs
                 try:
                     # log_file might be a relative path, in which case we need
@@ -200,9 +217,9 @@ def _server_info(server_val, get_defaults=False, options=None):
                     params_dict['warnings'].append(warning_msg)
 
         else:
-            params_dict['warnings'].append("Unable to get information "
-                                           "regarding variable '{0}'"
-                                           ).format(msg)
+            params_dict['warnings'].append(
+                "Unable to get information regarding variable '{0}'"
+            ).format(msg)
 
     # if audit_log plugin is installed and enabled
     if server.supports_plugin('audit'):
@@ -249,16 +266,15 @@ def _server_info(server_val, get_defaults=False, options=None):
     defaults = []
     if get_defaults:
         # Can only get defaults for local servers (need to access local data).
-        if server.is_alias('localhost'):
+        if server_is_local:
             try:
                 my_def_path = get_tool_path(params_dict['basedir'],
                                             "my_print_defaults", quote=True)
             except UtilError as err:
                 raise UtilError("Unable to retrieve the defaults data "
                                 "(requires access to my_print_defaults): {0} "
-                                "(basedir: {1})".format(err.errmsg,
-                                                        params_dict['basedir'])
-                                )
+                                "(basedir: {1})"
+                                "".format(err.errmsg, params_dict['basedir']))
             out_file = tempfile.TemporaryFile()
             # Execute tool: <basedir>/my_print_defaults mysqld
             cmd_list = shlex.split(my_def_path)
@@ -366,8 +382,10 @@ def _start_server(server_val, basedir, datadir, options=None):
         args.extend(server_args)
 
     socket = server_val.get('unix_socket', None)
+    if not socket and post_5_7_4 and os.name == "posix":
+        socket = os.path.normpath(os.path.join(datadir, "mysql.sock"))
     if socket is not None:
-        args.append("--socket=%(unix_socket)s" % server_val)
+        args.append("--socket={0}".format(socket))
     if verbosity > 0:
         subprocess.Popen(args, shell=False)
     else:
@@ -525,6 +543,9 @@ def show_server_info(servers, options):
             _show_running_servers(start_p, end_p)
         else:
             _show_running_servers()
+        # Don't continue unless at least one server is specified.
+        if not servers:
+            return
 
     ssl_dict = {}
     ssl_dict['ssl_cert'] = options.get("ssl_cert", None)
@@ -558,16 +579,16 @@ def show_server_info(servers, options):
             # If we got errno 2003 and we do not have
             # socket, instead we check if server is localhost.
             elif (util_error.errno == CR_CONN_HOST_ERROR and
-                    server1.is_alias("localhost")):
+                  server1.is_alias("localhost")):
                 server_is_off = True
             # If we got errno 1045 it means Access denied,
             # notify the user if a password was used or not.
             elif util_error.errno == ER_ACCESS_DENIED_ERROR:
                 use_pass = 'YES' if conn_dict['passwd'] else 'NO'
-                err_msg = ("Access denied for user '{0}'@'{1}' using "
+                err_msg = ["Access denied for user '{0}'@'{1}' using "
                            "password: {2}".format(conn_dict['user'],
                                                   conn_dict['host'],
-                                                  use_pass))
+                                                  use_pass)]
             # Use the error message from the connection attempt.
             else:
                 err_msg = [util_error.errmsg]
