@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,10 +19,6 @@
 This module contains classes and functions used to manage a console utility.
 """
 
-_COMMAND_COMPLETE = 0
-_OPTION_COMPLETE = 1
-_VARIABLE_COMPLETE = 2
-
 import os
 import sys
 import shlex
@@ -31,6 +27,10 @@ from mysql.utilities.common.format import print_dictionary_list
 from mysql.utilities.common.variables import Variables
 from mysql.utilities.exception import UtilError
 
+_COMMAND_COMPLETE = 0
+_OPTION_COMPLETE = 1
+_VARIABLE_COMPLETE = 2
+
 # TODO remove this pylint disable regarding duplicate keys
 # pylint: disable=W0109
 _COMMAND_KEY = {
@@ -46,7 +46,9 @@ _COMMAND_KEY = {
     '\t': 'TAB',
     '\x7f': 'BACKSPACE_POSIX',
     '\xe0': 'SPECIAL_WIN',
-    '\x08': 'BACKSPACE_WIN'
+    '\x08': 'BACKSPACE_WIN',
+    '\x1bOH': 'HOME',
+    '\x1bOF': 'END'
 }
 
 # Some windows keys are different and require reading two keys.
@@ -56,7 +58,9 @@ _WIN_COMMAND_KEY = {
     'H': 'ARROW_UP',
     'P': 'ARROW_DN',
     'M': 'ARROW_RT',
-    'K': 'ARROW_LT'
+    'K': 'ARROW_LT',
+    'G': 'HOME',
+    'O': 'END'
 }
 
 _COMMAND_COMPLETE = 0
@@ -98,11 +102,11 @@ _BASE_COMMANDS = [
     {'name': '<TAB>',
      'alias': '',
      'text': 'Press TAB for type completion of utility, '
-     'option, or variable names.'},
+             'option, or variable names.'},
     {'name': '<TAB><TAB>',
      'alias': '',
      'text': 'Press TAB twice for list of matching type '
-     'completion (context sensitive).'}
+             'completion (context sensitive).'}
 ]
 
 
@@ -110,9 +114,11 @@ _BASE_COMMANDS = [
 # a custom getch() method to return keys.
 try:
     # Win32
+    # pylint: disable=C0413
     from msvcrt import getch  # pylint: disable=F0401
 except ImportError:
     # UNIX/Posix
+    # pylint: disable=C0411,C0413
     import termios
 
     def getch():
@@ -281,6 +287,23 @@ class _Command(object):
         elif backspace:
             self._erase_portion(1)
 
+    def home_keypress(self):
+        """Executes the 'HOME' key press.
+
+        This moves the cursor to the beginning of the command.
+        """
+        tmp = self.position
+        self.position = 0
+        sys.stdout.write('\b' * tmp)
+
+    def end_keypress(self):
+        """Executes the 'END' key press.
+
+        This moves the cursor to the end of the command.
+        """
+        sys.stdout.write(self.command[self.position:self.length])
+        self.position = self.length
+
     def delete_keypress(self):
         """Execute the 'DELETE' key press.
 
@@ -300,8 +323,11 @@ class _Command(object):
                     self.command += old_command[self.position + 1:]
                 sys.stdout.write(self.command[self.position:])
                 self.length = len(self.command)
-                if self.length > 0:
+                spaces = len(self.command[self.position:])
+                # pylint: disable=W0612
+                for i in range(0, spaces):
                     sys.stdout.write('\b')
+                sys.stdout.flush()
 
     def backspace_keypress(self):
         """Execute the 'BACKSPACE' key press.
@@ -810,6 +836,10 @@ class Console(object):
             self.cmd_line.right_arrow_keypress()
         elif cmd_key in ['BACKSPACE_POSIX', 'BACKSPACE_WIN']:
             self.cmd_line.backspace_keypress()
+        elif cmd_key == 'HOME':
+            self.cmd_line.home_keypress()
+        elif cmd_key == 'END':
+            self.cmd_line.end_keypress()
         else:  # 'TAB'
             segment = self._set_complete_mode()
             self.tab_count += 1
@@ -903,7 +933,7 @@ class Console(object):
             if key in _COMMAND_KEY:
                 cmd_key = _COMMAND_KEY[key]
                 # Windows does things oddly for some keys
-                if not os.name == 'posix' and cmd_key == 'SPECIAL_WIN':
+                if os.name != 'posix' and cmd_key == 'SPECIAL_WIN':
                     key = getch()
                     cmd_key = _WIN_COMMAND_KEY.get(key)
                     if cmd_key is None:
