@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,10 +24,14 @@ import tempfile
 import difflib
 
 from mysql.utilities.exception import UtilError, UtilDBError
-from mysql.utilities.common.format import print_list
-from mysql.utilities.common.pattern_matching import REGEXP_QUALIFIED_OBJ_NAME
+from mysql.utilities.common.format import print_list, get_col_widths
+from mysql.utilities.common.pattern_matching import (
+    parse_object_name,
+    REGEXP_QUALIFIED_OBJ_NAME,
+    REGEXP_QUALIFIED_OBJ_NAME_AQ)
 from mysql.utilities.common.database import Database
 from mysql.utilities.common.lock import Lock
+from mysql.utilities.common.options import PARSE_ERR_OBJ_NAME_FORMAT
 from mysql.utilities.common.server import connect_servers
 from mysql.utilities.common.table import Table
 from mysql.utilities.common.sql_transform import (is_quoted_with_backticks,
@@ -107,7 +111,7 @@ _WARNING_INDEX_NOT_USABLE = ("# Warning: Specified index {idx} for table {tb}"
                              " cannot be used. It contains at least one "
                              "column that accepts null values.")
 
-_RE_EMPTY_ALTER_TABLE = "^ALTER TABLE {0};$".format(REGEXP_QUALIFIED_OBJ_NAME)
+_RE_EMPTY_ALTER_TABLE = "^ALTER TABLE {0};$"
 
 _RE_DASHES_DIG = re.compile(r"^\-{3}\s\d+")
 
@@ -154,10 +158,15 @@ def get_create_object(server, object_name, options, object_type):
     verbosity = options.get("verbosity", 0)
     quiet = options.get("quiet", False)
 
-    m_obj = re.match(REGEXP_QUALIFIED_OBJ_NAME, object_name)
-    db_name, obj_name = m_obj.groups()
+    # Get the sql_mode set on server
+    sql_mode = server.select_variable("SQL_MODE")
+
+    db_name, obj_name = parse_object_name(object_name, sql_mode)
     obj = [db_name]
 
+    if db_name is None:
+        raise UtilError(PARSE_ERR_OBJ_NAME_FORMAT.format(
+            obj_name=object_name, option=object_type.lower()))
     db = Database(server, obj[0], options)
 
     # Error if database does not exist
@@ -174,11 +183,11 @@ def get_create_object(server, object_name, options, object_type):
     if verbosity > 0 and not quiet:
         if obj_name:
             print("\n# Definition for object {0}.{1}:"
-                  "".format(remove_backtick_quoting(db_name),
-                            remove_backtick_quoting(obj_name)))
+                  "".format(remove_backtick_quoting(db_name, sql_mode),
+                            remove_backtick_quoting(obj_name, sql_mode)))
         else:
             print("\n# Definition for object {0}:"
-                  "".format(remove_backtick_quoting(db_name)))
+                  "".format(remove_backtick_quoting(db_name, sql_mode)))
         print create_stmt
 
     return create_stmt
@@ -239,7 +248,9 @@ def server_connect(server1_val, server2_val, object1, object2, options):
     if server2 is None:
         server2 = server1
 
-    if server1 == server2 and object1 == object2:
+    # Check if the specified objects and servers are the same.
+    if object1 == object2 and server1.port == server2.port and \
+       server1.is_alias(server2.host):
         raise UtilError("Comparing the same object on the same server.")
 
     return (server1, server2)
@@ -386,10 +397,11 @@ def _get_transform(server1, server2, object1, object2, options,
     """
 
     try:
-        m_obj1 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object1)
-        db1, name1 = m_obj1.groups()
-        m_obj2 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object2)
-        db2, name2 = m_obj2.groups()
+        db1, name1 = parse_object_name(object1,
+                                       server1.select_variable("SQL_MODE"))
+
+        db2, name2 = parse_object_name(object2,
+                                       server2.select_variable("SQL_MODE"))
     except:
         raise UtilError("Invalid object name arguments for _get_transform"
                         "(): %s, %s." % (object1, object2))
@@ -443,10 +455,11 @@ def _check_tables_structure(server1, server2, object1, object2, options,
     the third is a boolean indicating if the partition options are the same.
     """
     try:
-        m_obj1 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object1)
-        db1, name1 = m_obj1.groups()
-        m_obj2 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object2)
-        db2, name2 = m_obj2.groups()
+        db1, name1 = parse_object_name(object1,
+                                       server1.select_variable("SQL_MODE"))
+
+        db2, name2 = parse_object_name(object2,
+                                       server2.select_variable("SQL_MODE"))
     except:
         raise UtilError("Invalid object name arguments for diff_objects(): "
                         "{0}, {1}.".format(object1, object2))
@@ -499,7 +512,7 @@ def build_diff_list(diff1, diff2, transform1, transform2,
     Note: to specify a non-SQL difference for data, set
           options['data_diff'] = True
 
-    diff1[in]              definitiion diff for first server
+    diff1[in]              definition diff for first server
     diff2[in]              definition diff for second server
     transform1[in]         transformation for first server
     transform2[in]         transformation for second server
@@ -590,6 +603,20 @@ def diff_objects(server1, server2, object1, object2, options, object_type):
     object1_create = get_create_object(server1, object1, options, object_type)
     object2_create = get_create_object(server2, object2, options, object_type)
 
+    # Only target CREATE DATABASE difference if decorations differ,
+    # not just the database names. So we isolate the CREATE statement
+    # without the names or +/- and compare. If different, print the
+    # difference report otherwise, ignore it.
+    if (object_type == "DATABASE") and (object1 != object2):
+        quotes = ["'", '"', "`"]
+        db1 = object1.translate(None, "".join(quotes))
+        db2 = object2.translate(None, "".join(quotes))
+        first = object1_create.replace(db1, "")[1::]
+        second = object2_create.replace(db2, "")[1::]
+        if first == second:
+            object1_create = ""
+            object2_create = ""
+
     if not quiet:
         msg = "# Comparing {0} to {1} ".format(object1, object2)
         print msg,
@@ -649,8 +676,12 @@ def diff_objects(server1, server2, object1, object2, options, object_type):
     # Check if ALTER TABLE statement have changes. If not, it is probably
     # because there are differences but they have no influence on the create
     # table, such as different order on indexes.
-    if (diff_list and same_tbl_def and same_part_def
-       and re.match(_RE_EMPTY_ALTER_TABLE, diff_list[1])):
+    if "ANSI_QUOTES" in server1.select_variable("SQL_MODE"):
+        regex_pattern = REGEXP_QUALIFIED_OBJ_NAME_AQ
+    else:
+        regex_pattern = _RE_EMPTY_ALTER_TABLE.format(REGEXP_QUALIFIED_OBJ_NAME)
+    if diff_list and same_tbl_def and same_part_def and \
+       re.match(regex_pattern, diff_list[1]):
         print("[PASS]")
         return None
 
@@ -664,10 +695,10 @@ def diff_objects(server1, server2, object1, object2, options, object_type):
 
     # Check for failure to generate SQL statements
     if (difftype == 'sql') and \
-       ((direction == 'server1' and transform_server1 == []
-         and diff_server1 != []) or
-        (direction == 'server2' and transform_server2 == []
-         and diff_server2 != [])):
+       ((direction == 'server1' and transform_server1 == [] and
+         diff_server1 != []) or
+        (direction == 'server2' and transform_server2 == [] and
+         diff_server2 != [])):
 
         # Here we found no transformations. So either the change is nothing
         # more than the database name or we missed something. Send a
@@ -721,14 +752,15 @@ def _drop_compare_object(server, db_name, tbl_name):
     tbl_name[in]           table name
     """
     # Quote compare table appropriately with backticks
-    q_db_name = db_name if is_quoted_with_backticks(db_name) \
-        else quote_with_backticks(db_name)
-    if is_quoted_with_backticks(tbl_name):
-        q_tbl_name = remove_backtick_quoting(tbl_name)
+    sql_mode = server.select_variable("SQL_MODE")
+    q_db_name = db_name if is_quoted_with_backticks(db_name, sql_mode) \
+        else quote_with_backticks(db_name, sql_mode)
+    if is_quoted_with_backticks(tbl_name, sql_mode):
+        q_tbl_name = remove_backtick_quoting(tbl_name, sql_mode)
     else:
         q_tbl_name = tbl_name
     q_tbl_name = quote_with_backticks(
-        _COMPARE_TABLE_NAME.format(tbl=q_tbl_name))
+        _COMPARE_TABLE_NAME.format(tbl=q_tbl_name), sql_mode)
 
     try:
         # set autocommit=1 if it is 0, because CREATE TEMPORARY TABLE and
@@ -765,17 +797,19 @@ def _get_compare_objects(index_cols, table1,
     table = None
 
     # build primary key col definition
-    index_str = ''.join("{0}, ".format(quote_with_backticks(col[0]))
+    index_str = ''.join("{0}, ".format(quote_with_backticks(col[0],
+                                                            table1.sql_mode))
                         for col in index_cols)
     index_defn = ''.join("{0} {1}, ".
-                         format(quote_with_backticks(col[0]), col[1])
+                         format(quote_with_backticks(col[0], table1.sql_mode),
+                                col[1])
                          for col in index_cols)
     if index_defn == "":
         raise UtilError("Cannot generate index definition")
     else:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name), table1.sql_mode)
 
         table = _COMPARE_TABLE.format(db=table1.q_db_name,
                                       compare_tbl=q_tbl_name,
@@ -905,7 +939,7 @@ def _setup_compare(table1, table2, span_key_size, use_indexes=None):
     # Build the primary key hash if needed
     tbl1_table, pri_idx1 = _get_compare_objects(table1_idx, table1,
                                                 span_key_size)
-    tbl2_table, pri_idx2 = _get_compare_objects(table1_idx, table2,
+    tbl2_table, pri_idx2 = _get_compare_objects(table2_idx, table2,
                                                 span_key_size)
 
     if tbl1_table is None or tbl2_table is None:
@@ -961,7 +995,9 @@ def _make_sum_rows(table, idx_str, span_key_size=8):
 
     # Quote compare table appropriately with backticks
     q_tbl_name = quote_with_backticks(
-        _COMPARE_TABLE_NAME.format(tbl=table.tbl_name))
+        _COMPARE_TABLE_NAME.format(tbl=table.tbl_name),
+        table.sql_mode
+    )
 
     table.server.exec_query(
         _COMPARE_INSERT.format(db=table.q_db_name, compare_tbl=q_tbl_name,
@@ -1001,7 +1037,9 @@ def _get_rows_span(table, span, index):
     for row in span:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table.tbl_name),
+            table.sql_mode
+        )
 
         span_rows = server.exec_query(
             _COMPARE_DIFF.format(db=table.q_db_name, compare_tbl=q_tbl_name,
@@ -1049,7 +1087,9 @@ def _get_changed_rows_span(table1, table2, span, index):
     for row in span:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name),
+            table1.sql_mode
+        )
 
         span_rows = server1.exec_query(
             _COMPARE_DIFF.format(db=table1.q_db_name, compare_tbl=q_tbl_name,
@@ -1065,7 +1105,9 @@ def _get_changed_rows_span(table1, table2, span, index):
     for row in span:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table2.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table2.tbl_name),
+            table2.sql_mode
+        )
 
         span_rows = server2.exec_query(
             _COMPARE_DIFF.format(db=table2.q_db_name, compare_tbl=q_tbl_name,
@@ -1145,7 +1187,7 @@ def _get_changed_rows_span(table1, table2, span, index):
     return (changed_in1, extra_in1), (changed_in2, extra_in2)
 
 
-def _get_formatted_rows(rows, table, fmt='GRID'):
+def _get_formatted_rows(rows, table, fmt='GRID', col_widths=None):
     """Get a printable representation of the data rows
 
     This method generates a formatted view of the rows from a table. The output
@@ -1158,15 +1200,19 @@ def _get_formatted_rows(rows, table, fmt='GRID'):
     obj1_str[in]      full table name for base table
     obj2_str[in]      full table name for other table
     fmt[in]           format to print
+    col_widths[in]    column widths to use instead of actual col
 
     Returns list of formatted rows
     """
     result_rows = []
+    if not col_widths:
+        col_widths = []
     outfile = tempfile.TemporaryFile()
     to_sql = False
     if fmt.upper() == 'CSV':
         to_sql = True
-    print_list(outfile, fmt, table.get_col_names(), rows, to_sql=to_sql)
+    print_list(outfile, fmt, table.get_col_names(), rows, to_sql=to_sql,
+               col_widths=col_widths)
     outfile.seek(0)
     for line in outfile.readlines():
         result_rows.append(line.strip('\n'))
@@ -1202,6 +1248,37 @@ def _generate_data_diff_output(diff_data, table1, table2, used_index, options):
     changed_rows, extra1, extra2 = diff_data
     data_diffs = []
 
+    def get_max_cols(tbl1_rows, tbl2_rows):
+        """Get maximum columns for each set of rows
+
+        Find maximum column widths for each column for a pair of tables.
+
+        tbl1_rows[in]  first table rows
+        tbl2_rows[in]  second table rows
+
+        Return a list of the columns and the max width for each
+        """
+        # We need to turn the list of tuples to list of lists
+        row_list = []
+        for r in tbl1_rows:
+            for i in r:
+                row_list.append(list(i))
+        t1_cols = get_col_widths(table1.get_col_names(), row_list)
+        row_list = []
+        for r in tbl2_rows:
+            for i in r:
+                row_list.append(list(i))
+        t2_cols = get_col_widths(table2.get_col_names(), row_list)
+
+        # Get max of each
+        max_cols = []
+        for i in range(0, len(t1_cols)):
+            if t1_cols[i] > t2_cols[i]:
+                max_cols.append(t1_cols[i])
+            elif t1_cols[i] <= t2_cols[i]:
+                max_cols.append(t2_cols[i])
+        return max_cols
+
     if len(changed_rows) > 0:
         data_diffs.append("# Data differences found among rows:")
         # Get original changed/extra rows for each table within the given
@@ -1224,12 +1301,17 @@ def _generate_data_diff_output(diff_data, table1, table2, used_index, options):
                 data_diffs.extend(transform_data(table1, table2,
                                                  "INSERT", tbl2_rows[1]))
         else:
+            # Ok, to make the comparison more uniform, we need to get the
+            # max column widths for each table and use the higher of the
+            # two to format the rows in the output.
+            max_cols = get_max_cols(tbl1_rows, tbl2_rows)
+
             # Join changed and extra rows for table 1.
             tbl1_rows = tbl1_rows[0] + tbl1_rows[1]
-            rows1 = _get_formatted_rows(tbl1_rows, table1, fmt)
+            rows1 = _get_formatted_rows(tbl1_rows, table1, fmt, max_cols)
             # Join changed and extra rows for table 2.
             tbl2_rows = tbl2_rows[0] + tbl2_rows[1]
-            rows2 = _get_formatted_rows(tbl2_rows, table2, fmt)
+            rows2 = _get_formatted_rows(tbl2_rows, table2, fmt, max_cols)
             # Compute diff for all changes between table 1 and 2.
             diff_str = _get_diff(rows1, rows2, table1_name, table2_name,
                                  difftype, compact=compact_diff)
@@ -1389,18 +1471,32 @@ def check_consistency(server1, server2, table1_name, table2_name,
         if reporter:
             reporter.report_state("SKIP")
 
+    # remove quotations to indexes
+    unq_use_indexes = []
+    if use_indexes:
+        for tbl, index in use_indexes:
+            unq_use_indexes.append((
+                remove_backtick_quoting(tbl, table1.sql_mode),
+                remove_backtick_quoting(index, table1.sql_mode)
+            ))
+            if table1.sql_mode != table2.sql_mode:
+                unq_use_indexes.append((
+                    remove_backtick_quoting(tbl, table2.sql_mode),
+                    remove_backtick_quoting(index, table2.sql_mode)
+                ))
+
     # if given get the unique_key for table_name
     table1_use_indexes = []
     if use_indexes:
         table1_use_indexes.extend(
-            [u_key for tb_name, u_key in use_indexes
-             if table1_name.endswith(".{0}".format(tb_name))]
+            [quote_with_backticks(u_key, table1.sql_mode) for tb_name, u_key
+             in unq_use_indexes if table1.tbl_name == tb_name]
         )
     table2_use_indexes = []
     if use_indexes:
         table2_use_indexes.extend(
-            [u_key for tb_name, u_key in use_indexes
-             if table2_name.endswith(".{0}".format(tb_name))]
+            [quote_with_backticks(u_key, table2.sql_mode) for tb_name, u_key
+             in unq_use_indexes if table2.tbl_name == tb_name]
         )
 
     if options.get('toggle_binlog', 'False'):
@@ -1430,7 +1526,7 @@ def check_consistency(server1, server2, table1_name, table2_name,
     pri_idx_str1, pri_idx_str2, used_index, used_index_name, msgs = (
         _setup_compare(table1, table2,
                        span_key_size,
-                       use_indexes=(table2_use_indexes, table2_use_indexes))
+                       use_indexes=(table1_use_indexes, table2_use_indexes))
     )
 
     # Add warnings to print them later.
