@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,9 +24,16 @@ METHODS
                           Writes to a file specified (e.g. sys.stdout)
 """
 
+import codecs
 import csv
 import os
 import textwrap
+
+try:
+    import cStringIO as StringIO
+except ImportError:
+    import StringIO
+
 from mysql.utilities.common.sql_transform import to_sql
 
 
@@ -34,6 +41,46 @@ _MAX_WIDTH = 78
 _TWO_COLUMN_DISPLAY = "{0:{1}}  {2:{3}}"
 
 
+class UnicodeWriter(object):
+    """A CSV writer which will write rows to CSV file `f_out`,
+    which is encoded in the given encoding.
+    """
+
+    def __init__(self, f_out, dialect="excel", encoding="utf-8", **kwds):
+        """Contructor
+
+        f_out[in]        file to print to (e.g. sys.stdout)
+        dialect[in]      description of the dialect in use by the writer
+        encoding[in]     encoding
+        """
+        # Redirect output to a queue
+        self.queue = StringIO.StringIO()
+        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
+        self.stream = f_out
+        self.encoder = codecs.getincrementalencoder(encoding)()
+
+    def writerow(self, row):
+        """Write the row parameter to the writer's file object.
+
+        row[in]     sequence of strings or numbers
+        """
+        self.writer.writerow([val.encode("utf-8") if isinstance(val, unicode)
+                              else val for val in row])
+        data = self.queue.getvalue()
+        data = data.decode("utf-8")  # pylint: disable=R0204
+        data = self.encoder.encode(data)
+        self.stream.write(data)
+        self.queue.truncate(0)
+
+    def writerows(self, rows):
+        """Write all rows parameter to the writer's file object.
+
+        rows[in]     list of row objects
+        """
+        for row in rows:
+            self.writerow(row)
+
+
 def _format_col_separator(f_out, columns, col_widths, quiet=False):
     """Format a row of the header with column separators
 
@@ -66,12 +113,52 @@ def _format_row_separator(f_out, columns, col_widths, row, quiet=False):
     for i, _ in enumerate(columns):
         if not quiet:
             f_out.write("| ")
-        f_out.write("{0:<{1}} ".format("%s" % row[i], col_widths[i]))
+        val = row[i].encode("utf-8") if isinstance(row[i], unicode) \
+            else row[i]
+        if isinstance(val, str):
+            val = u"{0:<{1}}".format(val.decode("utf-8"), col_widths[i] + 1)
+            f_out.write(val.encode("utf-8"))
+        else:
+            f_out.write("{0:<{1}} ".format("%s" % val, col_widths[i]))
+
     if not quiet:
         f_out.write("|")
     f_out.write("\n")
 
 
+def get_col_widths(columns, rows):
+    """
+    This function gets the maximum column width for a list of rows
+
+    Returns: list - max column widths
+    """
+    # Calculate column width for each column
+    col_widths = []
+    for col in columns:
+        size = len(col.decode("utf-8") if isinstance(col, str) else col)
+        col_widths.append(size + 1)
+
+    stop = len(columns)
+    for row in rows:
+        row = [val.encode("utf-8") if isinstance(val, unicode)
+               else val for val in row]
+        # if there is one column, just use row.
+        if stop == 1:
+            col_size = len(row[0].decode("utf-8")
+                           if isinstance(row[0], str) else str(row[0]))
+            col_size += 1
+            if col_size > col_widths[0]:
+                col_widths[0] = col_size
+        else:
+            for i in range(0, stop):
+                col_size = len(row[i].decode("utf-8")
+                               if isinstance(row[i], str) else str(row[i]))
+                col_size += 1
+                if col_size > col_widths[i]:
+                    col_widths[i] = col_size
+    return col_widths
+
+
 def format_tabular_list(f_out, columns, rows, options=None):
     """Format a list in a pretty grid format.
 
@@ -101,13 +188,17 @@ def format_tabular_list(f_out, columns, rows, options=None):
         return
     if separator is not None:
         if os.name == "posix":
-            csv_writer = csv.writer(f_out, delimiter=separator)
+            # Use \n as line terminator in POSIX (non-Windows) systems.
+            csv_writer = UnicodeWriter(f_out, delimiter=separator,
+                                       lineterminator='\n')
         else:
-            csv_writer = csv.writer(f_out, delimiter=separator,
-                                    lineterminator='\n')
+            # Use the default line terminator '\r\n' on Windows.
+            csv_writer = UnicodeWriter(f_out, delimiter=separator)
         if print_header:
             csv_writer.writerow(columns)
         for row in rows:
+            row = [val.encode("utf-8") if isinstance(val, unicode)
+                   else val for val in row]
             if convert_to_sql:
                 # Convert value to SQL (i.e. add quotes if needed).
                 row = ['NULL' if col is None else to_sql(col) for col in row]
@@ -117,23 +208,9 @@ def format_tabular_list(f_out, columns, rows, options=None):
             csv_writer.writerow(row)
     else:
         # Calculate column width for each column
-        col_widths = []
-        for col in columns:
-            size = len(col)
-            col_widths.append(size + 1)
-
-        stop = len(columns)
-        for row in rows:
-            # if there is one column, just use row.
-            if stop == 1:
-                col_size = len(str(row[0])) + 1
-                if col_size > col_widths[0]:
-                    col_widths[0] = col_size
-            else:
-                for i in range(0, stop):
-                    col_size = len(str(row[i])) + 1
-                    if col_size > col_widths[i]:
-                        col_widths[i] = col_size
+        col_widths = options.get('col_widths', None)
+        if not col_widths:
+            col_widths = get_col_widths(columns, rows)
 
         # print header
         if print_header:
@@ -192,8 +269,12 @@ def format_vertical_list(f_out, columns, rows, options=None):
             # Convert None values to 'NULL'
             row = ['NULL' if not val else val for val in row]
         for i in range(0, stop):
-            f_out.write("{0:>{1}}: {2}\n".format(columns[i], max_colwidth,
-                                                 row[i]))
+            col = columns[i].decode("utf-8") \
+                if isinstance(columns[i], str) else columns[i]
+            val = row[i].decode("utf-8") \
+                if isinstance(row[i], str) else row[i]
+            out = u"{0:>{1}}: {2}\n".format(col, max_colwidth, val)
+            f_out.write(out.encode("utf-8"))
 
     if row_num > 0:
         row_str = 'rows' if row_num > 1 else 'row'
@@ -201,7 +282,7 @@ def format_vertical_list(f_out, columns, rows, options=None):
 
 
 def print_list(f_out, fmt, columns, rows, no_headers=False, sort=False,
-               to_sql=False):
+               to_sql=False, col_widths=None):
     """Print a list< based on format.
 
     Prints a list of rows in the format chosen. Default is GRID.
@@ -214,13 +295,17 @@ def print_list(f_out, fmt, columns, rows, no_headers=False, sort=False,
     sort[in]          If True, sort list before printing
     to_sql[out]       If True, converts columns to SQL format before
                       printing them to the output.
+    col_widths[in]    col widths to use instead of actual col
     """
 
+    if not col_widths:
+        col_widths = []
     if sort:
         rows.sort()
     list_options = {
         'print_header': not no_headers,
-        'to_sql': to_sql
+        'to_sql': to_sql,
+        'col_widths': col_widths,
     }
     if fmt == "vertical":
         format_vertical_list(f_out, columns, rows)
@@ -243,7 +328,9 @@ def _get_max_key_dict_list(dictionary_list, key, alias_key=None):
 
     Returns int - max width of key
     """
-    lcal = lambda x: len(str(x or ''))
+    def lcal(x):
+        """ calculate string length """
+        return len(str(x or ''))
     dl = dictionary_list
     tmp = [(lcal(item[key]), lcal(item.get(alias_key, 0))) for item in dl]
     return max([(x[0] + x[1] + 3) if x[1] else x[0] for x in tmp])
