@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,19 +25,25 @@ Methods:
 
 import copy
 import optparse
+from optparse import Option as CustomOption, OptionValueError
 import os.path
 import re
 
-from optparse import Option as CustomOption, OptionValueError
-from ip_parser import find_password
-
+from datetime import datetime
+from ip_parser import find_password, parse_login_values_config_path
 from mysql.utilities import LICENSE_FRM, VERSION_FRM
 from mysql.utilities.exception import UtilError, FormatError
 from mysql.connector.conversion import MySQLConverter
-from mysql.utilities.common.messages import PARSE_ERR_OBJ_NAME_FORMAT
+from mysql.utilities.common.messages import (PARSE_ERR_OBJ_NAME_FORMAT,
+                                             PARSE_ERR_OPT_INVALID_DATE,
+                                             PARSE_ERR_OPT_INVALID_DATE_TIME,
+                                             PARSE_ERR_OPT_INVALID_NUM_DAYS,
+                                             PARSE_ERR_OPT_INVALID_VALUE,
+                                             EXTERNAL_SCRIPT_DOES_NOT_EXIST,
+                                             INSUFFICIENT_FILE_PERMISSIONS)
 from mysql.utilities.common.my_print_defaults import (MyDefaultsReader,
                                                       my_login_config_exists)
-from mysql.utilities.common.pattern_matching import REGEXP_QUALIFIED_OBJ_NAME
+from mysql.utilities.common.pattern_matching import parse_object_name
 from mysql.utilities.common.sql_transform import (is_quoted_with_backticks,
                                                   remove_backtick_quoting)
 
@@ -121,6 +127,22 @@ def path_callback(option, opt, value, parser):
     setattr(parser.values, option.dest, value)
 
 
+def ssl_callback(option, opt, value, parser):
+    """Verify that the given path is an existing file. If it is then add it
+    to the parser values.
+
+    option[in]        option instance
+    opt[in]           option name
+    value[in]         given user value
+    parser[in]        parser instance
+    """
+    if not (value == 0 or value == 1 or value == ''):
+        parser.error("the given value '{0}' in option {1} is not"
+                     " valid, valid values are 0 or 1.".format(value, opt))
+
+    setattr(parser.values, option.dest, value)
+
+
 def add_config_path_option(parser):
     """Add the config_path option.
 
@@ -141,19 +163,25 @@ def add_ssl_options(parser):
     # --ssl options: ssl_ca, ssl_cert, ssl_key
     parser.add_option("--ssl-ca", action="callback",
                       callback=path_callback,
-                      type="string", help="The path to a file that contains "
+                      type="string", help="path to a file that contains "
                       "a list of trusted SSL CAs.")
 
     parser.add_option("--ssl-cert", action="callback",
                       callback=path_callback,
-                      type="string", help="The name of the SSL certificate "
+                      type="string", help="name of the SSL certificate "
                       "file to use for establishing a secure connection.")
 
     parser.add_option("--ssl-key", action="callback",
                       callback=path_callback,
-                      type="string", help="The name of the SSL key file to "
+                      type="string", help="name of the SSL key file to "
                       "use for establishing a secure connection.")
 
+    parser.add_option("--ssl", action="callback", callback=ssl_callback,
+                      type="int", help="specifies if the server "
+                      "connection requires use of SSL. If an encrypted "
+                      "connection cannot be established, the connection "
+                      "attempt fails. By default 0 (SSL not required).")
+
 
 class CaseInsensitiveChoicesOption(CustomOption):
     """Case insensitive choices option class
@@ -528,6 +556,48 @@ def add_locking(parser):
                       "transaction.")
 
 
+def add_exclude(parser, object_type="objects",
+                example1="db1.t1", example2="db1.t% or db%.%"):
+    """Add the --exclude option.
+
+    parser[in]        the parser instance
+    example1[in]
+    example2[in]
+    """
+    parser.add_option("-x", "--exclude", action="append", dest="exclude",
+                      type="string", default=None, help="exclude one or more "
+                      "{0} from the operation using either a specific "
+                      "name (e.g. {1}), a LIKE pattern (e.g. {2}) or a REGEXP "
+                      "search pattern. To use a REGEXP search pattern for all "
+                      "exclusions, you must also specify the --regexp option. "
+                      "Repeat the --exclude option for multiple exclusions."
+                      "".format(object_type, example1, example2))
+
+
+def check_exclude_pattern(exclude_list, use_regexp):
+    """Check the --exclude pattern to determine if there are special symbols
+    that may be regexp symbols and the --use-regexp option is not specified.
+    Prints warning if this is true.
+
+    parser[in]        the parser instance
+    use_regexp[in]    the option to use regexp
+    """
+    # ignore null lists
+    if not exclude_list:
+        return True
+    for row in exclude_list:
+        # replace _ and % and see if still not alnum()
+        test = row.replace('_', '').replace('%', '').replace('`', '')
+        test = test.replace("'", "").replace('.', '').replace('"', '')
+        if len(test) > 0 and not test.isalnum() and not use_regexp:
+            print "# WARNING: One or more of your --exclude patterns " \
+                  "contains symbols that could be regexp patterns. You may " \
+                  "need to include --regexp to ensure your exclude pattern " \
+                  "is evaluated as REGEXP and not a SQL LIKE expression."
+            return False
+    return True
+
+
 def add_regexp(parser):
     """Add the --regexp option.
 
@@ -672,7 +742,7 @@ def add_slaves_option(parser):
                       help="connection information for slave servers in "
                       "the form: <user>[:<password>]@<host>[:<port>]"
                       "[:<socket>] or <login-path>[:<port>][:<socket>]"
-                      " or <config-path>[<[group]>]."
+                      " or <config-path>[<[group]>]. "
                       "List multiple slaves in comma-separated list.")
 
 
@@ -784,13 +854,14 @@ def parse_user_password(userpass_values, my_defaults_reader=None,
     """ This function parses a string with the user/password credentials.
 
     This function parses the login string, determines the used format, i.e.
-    user[:password] or login-path. If the ':' (colon) is not in the login
-    string, the it can refer to a login-path or to a username (without a
-    password). In this case, first it is assumed that the specified value is a
-    login-path and the function attempts to retrieve the associated username
-    and password, in a quiet way (i.e., without raising exceptions). If it
-    fails to retrieve the login-path data, then the value is assumed to be a
-    username.
+    user[:password], config-path or login-path. If the ':' (colon) is not in
+    the login string, the it can refer to a config-path, login-path or to a
+    username (without a password). In this case, first it is assumed that the
+    specified value is a config-path and tries to retrive the user and password
+    from the configuration file secondly assume it is a login-path and the
+    function attempts to retrieve the associated username and password, in a
+    quiet way (i.e., without raising exceptions). If it fails to retrieve the
+    login-path data, then the value is assumed to be a username.
 
     userpass_values[in]     String indicating the user/password credentials. It
                             must be in the form: user[:password] or login-path.
@@ -806,16 +877,23 @@ def parse_user_password(userpass_values, my_defaults_reader=None,
     """
     if options is None:
         options = {}
-    # Split on the ':' to determine if a login-path is used.
-    login_values = userpass_values.split(':')
+    # Split on the first ':' to determine if a login-path is used.
+    login_values = userpass_values.split(':', 1)
     if len(login_values) == 1:
-        # Format is login-path or user (without a password): First, assume it
-        # is a login-path and quietly try to retrieve the user and password.
-        # If it fails, assume a user name is being specified.
+        # Format is config-path, login-path or user (without a password):
+        # First check if the value is a config-path
+        # The following method call also initializes the user and passwd with
+        # default values in case the login_values are not from a config-path
+        user, passwd = parse_login_values_config_path(login_values[0],
+                                                      quietly=True)
+
+        # Second assume it's a login-path and quietly try to retrieve the user
+        # and password, in case of success overwrite the values previously set
+        # and in case of failure return these ones instead.
 
         # Check if the login configuration file (.mylogin.cnf) exists
         if login_values[0] and not my_login_config_exists():
-            return login_values[0], None
+            return user, passwd
 
         if not my_defaults_reader:
             # Attempt to create the MyDefaultsReader
@@ -823,19 +901,19 @@ def parse_user_password(userpass_values, my_defaults_reader=None,
                 my_defaults_reader = MyDefaultsReader(options)
             except UtilError:
                 # Raise an UtilError when my_print_defaults tool is not found.
-                return login_values[0], None
+                return user, passwd
         elif not my_defaults_reader.tool_path:
             # Try to find the my_print_defaults tool
             try:
                 my_defaults_reader.search_my_print_defaults_tool()
             except UtilError:
                 # Raise an UtilError when my_print_defaults tool is not found.
-                return login_values[0], None
+                return user, passwd
 
         # Check if the my_print_default tool is able to read a login-path from
         # the mylogin configuration file
         if not my_defaults_reader.check_login_path_support():
-            return login_values[0], None
+            return user, passwd
 
         # Read and parse the login-path data (i.e., user and password)
         try:
@@ -845,19 +923,19 @@ def parse_user_password(userpass_values, my_defaults_reader=None,
                 passwd = loginpath_data.get('password', None)
                 return user, passwd
             else:
-                return login_values[0], None
+                return user, passwd
         except UtilError:
             # Raise an UtilError if unable to get the login-path group data
-            return login_values[0], None
+            return user, passwd
 
     elif len(login_values) == 2:
         # Format is user:password; return a tuple with the user and password
         return login_values[0], login_values[1]
     else:
         # Invalid user credentials format
-        return FormatError("Unable to parse the specified user credentials "
-                           "(accepted formats: <user>[:<password> or "
-                           "<login-path>): %s" % userpass_values)
+        raise FormatError("Unable to parse the specified user credentials "
+                          "(accepted formats: <user>[:<password> or "
+                          "<login-path>): %s" % userpass_values)
 
 
 def add_basedir_option(parser):
@@ -884,16 +962,53 @@ def check_dir_option(parser, opt_value, opt_name, check_access=False,
     read_only[in]       Flag indicating if the access required is only for
                         read or read/write. By default, False (read/write
                         access). Note: only used if check_access=True.
+
+    Return the absolute path for the specified directory or None if an empty
+    value is specified.
     """
     # Check existence of specified directory.
-    if opt_value and not os.path.isdir(get_absolute_path(opt_value)):
-        parser.error("The specified path for {0} option is not a "
-                     "directory: {1}".format(opt_name, opt_value))
-    if check_access:
-        mode = os.R_OK if read_only else os.R_OK | os.W_OK
-        if not os.access(opt_value, mode):
-            parser.error("You do not have enough privileges to access the "
-                         "folder specified by {0}.".format(opt_name))
+    if opt_value:
+        full_path = get_absolute_path(opt_value)
+        if not os.path.isdir(full_path):
+            parser.error("The specified path for {0} option is not a "
+                         "directory: {1}".format(opt_name, opt_value))
+        if check_access:
+            mode = os.R_OK if read_only else os.R_OK | os.W_OK
+            if not os.access(full_path, mode):
+                parser.error("You do not have enough privileges to access the "
+                             "folder specified by {0}.".format(opt_name))
+        return full_path
+    return None
+
+
+def check_script_option(parser, opt_value, check_executable=True):
+    """ Check if the specified script option is valid.
+
+    Check if the script specified for the option exists, and if
+    the user has appropriate access privileges to it. An appropriate parser
+    error is issued if the specified directory does not exist or is not
+    executable.
+
+    parser[in]            Instance of the option parser (optparse).
+    opt_value[in]         Value specified for the option.
+    check_executable[in]  Flag specifying if the executable privileges need to
+                          be checked. By default, True(needs to be executable).
+
+    Return the absolute path for the specified script or None if an empty
+    value is specified.
+    """
+    if opt_value:
+        abs_path = os.path.abspath(opt_value)
+        if not os.path.isfile(abs_path):
+            parser.error(EXTERNAL_SCRIPT_DOES_NOT_EXIST.format(
+                path=opt_value))
+
+        if check_executable and not os.access(abs_path, os.X_OK):
+            parser.error(INSUFFICIENT_FILE_PERMISSIONS.format(
+                path=opt_value, permissions='execute'))
+        return opt_value
+    else:
+        return None
 
 
 def get_absolute_path(path):
@@ -902,7 +1017,8 @@ def get_absolute_path(path):
     return os.path.abspath(os.path.expanduser(os.path.normpath(path)))
 
 
-def db_objects_list_to_dictionary(parser, obj_list, option_desc):
+def db_objects_list_to_dictionary(parser, obj_list, option_desc,
+                                  db_over_tables=True, sql_mode=''):
     """Process database object list and convert to a dictionary.
 
     Check the qualified name format of the given database objects and convert
@@ -912,11 +1028,16 @@ def db_objects_list_to_dictionary(parser, obj_list, option_desc):
     Note: It is assumed that the given object list is obtained from the
     arguments or an option returned by the parser.
 
-    parser[in]      Instance of the used option/arguments parser
-    obj_list[in]    List of objects to process.
-    option_desc[in] Short description of the option for the object list (e.g.,
-                    "the --exclude option", "the database/table arguments") to
-                    refer appropriately in any parsing error.
+    parser[in]            Instance of the used option/arguments parser
+    obj_list[in]          List of objects to process.
+    option_desc[in]       Short description of the option for the object list
+                          (e.g., "the --exclude option", "the database/table
+                          arguments") to refer appropriately in any parsing
+                          error.
+    db_over_tables[in]    If True specifying a db alone overrides all
+                          occurrences of table objects from that db (e.g.
+                          if True and we have both db and db.table1, db.table1
+                          is ignored).
 
     returns a dictionary with the objects grouped by database (without
     duplicates). None value associated to a database entry means that all
@@ -924,31 +1045,38 @@ def db_objects_list_to_dictionary(parser, obj_list, option_desc):
     E.g. {'db_name1': set(['table1','table2']), 'db_name2': None}.
     """
     db_objs_dict = {}
-    obj_name_regexp = re.compile(REGEXP_QUALIFIED_OBJ_NAME)
     for obj_name in obj_list:
-        m_obj = obj_name_regexp.match(obj_name)
-        if not m_obj:
+        m_objs = parse_object_name(obj_name, sql_mode)
+        if m_objs[0] is None:
             parser.error(PARSE_ERR_OBJ_NAME_FORMAT.format(
                 obj_name=obj_name, option=option_desc
             ))
         else:
-            db_name, obj_name = m_obj.groups()
+            db_name, obj_name = m_objs
             # Remove backtick quotes.
-            db_name = remove_backtick_quoting(db_name) \
-                if is_quoted_with_backticks(db_name) else db_name
-            obj_name = remove_backtick_quoting(obj_name) \
-                if obj_name and is_quoted_with_backticks(obj_name) \
+            db_name = remove_backtick_quoting(db_name, sql_mode) \
+                if is_quoted_with_backticks(db_name, sql_mode) else db_name
+            obj_name = remove_backtick_quoting(obj_name, sql_mode) \
+                if obj_name and is_quoted_with_backticks(obj_name, sql_mode) \
                 else obj_name
             # Add database object to result dictionary.
             if not obj_name:
-                # If only the database is specified, then add entry with
-                # db name and value None (to include all object) even if a
-                # previous specific object was already added.
+                # If only the database is specified and db_over_tables is True,
+                # then add entry with db name and value None (to include all
+                # objects) even if a previous specific object was already
+                # added, else if db_over_tables is False, add None value to the
+                #  list, so that we know db was specified without any
+                # table/routine.
                 if db_name in db_objs_dict:
-                    if db_objs_dict[db_name]:
+                    if db_objs_dict[db_name] and not db_over_tables:
+                        db_objs_dict[db_name].add(None)
+                    else:
                         db_objs_dict[db_name] = None
                 else:
-                    db_objs_dict[db_name] = None
+                    if db_over_tables:
+                        db_objs_dict[db_name] = None
+                    else:
+                        db_objs_dict[db_name] = set([None])
             else:
                 # If a specific object object is given add it to the set
                 # associated to the database, except if the database entry
@@ -982,10 +1110,164 @@ def get_ssl_dict(parser_options=None):
             certs_paths['ssl_cert'] = parser_options.ssl_cert
         if 'ssl_key' in dir(parser_options):
             certs_paths['ssl_key'] = parser_options.ssl_key
+        if 'ssl' in dir(parser_options):
+            certs_paths['ssl'] = parser_options.ssl
         conn_options.update(certs_paths)
     return conn_options
 
 
+def get_value_intervals_list(parser, option_value, option_name, value_name):
+    """Get and check the list of values for the given option.
+
+    Convert the string value for the given option to the corresponding
+    list of integer values and tuple of integers (for intervals). For example,
+    converts the option_value '3,5-8,11' to the list [3, (5,8), 11].
+
+    A parser error is issued if the used values or format are invalid.
+
+    parser[in]          Instance of the used option/arguments parser.
+    option_value[in]    Value specified for the option (e.g., '3,5-8,11').
+    option_name[in]     Name of the option (e.g., '--status').
+    value_name[in]      Name describing each option value (e.g., 'status').
+
+    Returns a list of integers and tuple of integers (for intervals)
+    representing the given option value string.
+    """
+    # Filter empty values and convert all to integers (values and intervals).
+    values = option_value.split(",")
+    values = [value for value in values if value]
+    if len(values) <= 0:
+        parser.error(PARSE_ERR_OPT_INVALID_VALUE.format(option=option_name,
+                                                        value=option_value))
+    res_list = []
+    for value in values:
+        interval = value.split('-')
+        if len(interval) == 2:
+            # Convert lower and higher value of the interval.
+            try:
+                lv = int(interval[0])
+            except ValueError:
+                parser.error("Invalid {0} value '{1}' (must be a "
+                             "non-negative integer) for interval "
+                             "'{2}'.".format(value_name, interval[0], value))
+            try:
+                hv = int(interval[1])
+            except ValueError:
+                parser.error("Invalid {0} value '{1}' (must be a "
+                             "non-negative integer) for interval "
+                             "'{2}'.".format(value_name, interval[1], value))
+            # Add interval (tuple) to the list.
+            res_list.append((lv, hv))
+        elif len(interval) == 1:
+            # Add single value to the status list.
+            try:
+                res_list.append(int(value))
+            except ValueError:
+                parser.error("Invalid {0} value '{1}' (must be a "
+                             "non-negative integer).".format(value_name,
+                                                             value))
+        else:
+            # Invalid format.
+            parser.error("Invalid format for {0} interval (a single "
+                         "dash must be used): '{1}'.".format(value_name,
+                                                             value))
+    return res_list
+
+
+def check_date_time(parser, date_value, date_type, allow_days=False):
+    """Check the date/time value for the given option.
+
+    Check if the date/time value for the option is valid. The supported
+    formats are 'yyyy-mm-ddThh:mm:ss' and 'yyyy-mm-dd'. If the allow days
+    flag is ON then an integer valuse representing the number of days is
+    also accepted.
+
+    A parser error is issued if the date/time value is invalid.
+
+    parser[in]        Instance of the used option/arguments parser.
+    date_value[in]    Date/time value specified for the option.
+    date_type[in]     Name describing the type of date being checked
+                      (e.g., start, end, modified).
+    allow_days[in]    Flag indicating if the specified value can also be an
+                      integer representing the number of of days (> 0).
+
+    Returns the date in the format 'yyyy-mm-ddThh:mm:ss' or an integer
+    representing the number of days.
+    """
+    if allow_days:
+        # Check if it is a valid number of days.
+        try:
+            days = int(date_value)
+        except ValueError:
+            # Not a valid integer (i.e., number of days).
+            days = None
+        if days:
+            if days < 1:
+                parser.error(PARSE_ERR_OPT_INVALID_NUM_DAYS.format(
+                    date_type, date_value))
+            return days
+    # Check if it is a valid date/time format.
+    _, _, time = date_value.partition("T")
+    if time:
+        try:
+            dt_date = datetime.strptime(date_value, '%Y-%m-%dT%H:%M:%S')
+        except ValueError:
+            parser.error(PARSE_ERR_OPT_INVALID_DATE_TIME.format(date_type,
+                                                                date_value))
+    else:
+        try:
+            dt_date = datetime.strptime(date_value, '%Y-%m-%d')
+        except ValueError:
+            parser.error(PARSE_ERR_OPT_INVALID_DATE.format(date_type,
+                                                           date_value))
+    return dt_date.strftime('%Y-%m-%dT%H:%M:%S')
+
+
+def check_gtid_set_format(parser, gtid_set):
+    """Check the format of the GTID set given for the option.
+
+    Perform some basic checks to verify the syntax of the specified string
+    for the GTID set value. A parse error is issued if the format is incorrect.
+
+    parser[in]      Instance of the used option/arguments parser.
+    gtid_set[in]    GTID set value specified for the option.
+    """
+
+    # UUID format: hhhhhhhh-hhhh-hhhh-hhhh-hhhhhhhhhhhh
+    re_uuid = re.compile(
+        r"(?:[a-f]|\d){8}(?:-(?:[a-f]|\d){4}){3}-(?:[a-f]|\d){12}",
+        re.IGNORECASE)
+    # interval format: n[-n]
+    re_interval = re.compile(r"(?:\d+)(?:-\d+)?")
+    uuid_sets = gtid_set.split(',')
+    for uuid_set in uuid_sets:
+        uuid_set_elements = uuid_set.split(':')
+        if len(uuid_set_elements) < 2:
+            parser.error("Invalid GTID set '{0}' for option --gtid-set, "
+                         "missing UUID or interval. Valid format: "
+                         "uuid:interval[:interval].".format(uuid_set))
+        # Check server UUID format.
+        if not re_uuid.match(uuid_set_elements[0]):
+            parser.error("Invalid UUID '{0}' for option --gtid-set. Valid "
+                         "format: hhhhhhhh-hhhh-hhhh-hhhh-hhhhhhhhhhhh."
+                         "".format(uuid_set_elements[0]))
+        # Check intervals.
+        for interval in uuid_set_elements[1:]:
+            if not re_interval.match(interval):
+                parser.error("Invalid interval '{0}' for option --gtid-set. "
+                             "Valid format: n[-n].".format(interval))
+            try:
+                start_val, end_val = interval.split('-')
+                if int(start_val) >= int(end_val):
+                    parser.error(
+                        "Invalid interval '{0}' for option --gtid-set. Start "
+                        "value must be lower than the end value."
+                        "".format(interval))
+            except ValueError:
+                # Error raised for intervals with a single value.
+                pass  # Ignore no need to compare start and end value.
+
+
 def check_password_security(options, args, prefix=""):
     """Check command line for passwords and report a warning.
 
@@ -1005,7 +1287,7 @@ def check_password_security(options, args, prefix=""):
     """
     result = False
     for value in options.__dict__.values():
-        if type(value) == list:
+        if isinstance(value, list):
             for item in value:
                 if find_password(item):
                     result = True
