@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,11 +25,13 @@ import StringIO
 import socket
 
 from mysql.utilities.exception import UtilError, UtilRplWarn, UtilRplError
+from mysql.utilities.exception import FormatError
 from mysql.utilities.common.options import parse_user_password
 from mysql.utilities.common.server import Server
 from mysql.utilities.common.user import User
 from mysql.utilities.common.ip_parser import clean_IPv6, format_IPv6
 from mysql.utilities.common.format import format_tabular_list
+from mysql.utilities.common.messages import USER_PASSWORD_FORMAT
 
 
 _MASTER_INFO_COL = [
@@ -56,13 +58,13 @@ _PRINT_WIDTH = 75
 _MASTER_DO_DB, _MASTER_IGNORE_DB = 2, 3
 
 _RPL_USER_QUERY = """
-    SELECT user, host, password = "" as has_password
+    SELECT user, host, password = '' as has_password
     FROM mysql.user
     WHERE repl_slave_priv = 'Y'
 """
 # Query for server versions >= 5.7.6.
 _RPL_USER_QUERY_5_7_6 = """
-    SELECT user, host, authentication_string = "" as has_password
+    SELECT user, host, authentication_string = '' as has_password
     FROM mysql.user
     WHERE repl_slave_priv = 'Y'
 """
@@ -144,6 +146,7 @@ def negotiate_rpl_connection(server, is_master=True, strict=True,
     change_master = []
 
     # If server is a master, perform error checking
+    # pylint: disable=R0101
     if is_master:
         master = Master(new_opts)
         master.connect()
@@ -160,7 +163,7 @@ def negotiate_rpl_connection(server, is_master=True, strict=True,
                     uname = ""
                     passwd = ""
                     # Throw error if strict but not for rpl_mode = both
-                    if strict and not rpl_mode == 'both':
+                    if strict and rpl_mode != 'both':
                         raise UtilRplError(_NO_RPL_USER)
                     else:
                         change_master.append(_WARNING % _NO_RPL_USER)
@@ -168,14 +171,18 @@ def negotiate_rpl_connection(server, is_master=True, strict=True,
                     uname = res[0][0]
                     if res[0][2]:
                         # Throw error if strict but not for rpl_mode = both
-                        if strict and not rpl_mode == 'both':
+                        if strict and rpl_mode != 'both':
                             raise UtilRplError(_RPL_USER_PASS)
                         else:
                             change_master.append(_WARNING % _RPL_USER_PASS)
                     passwd = res[0][1]
             else:
                 # Parse username and password (supports login-paths)
-                uname, passwd = parse_user_password(rpl_user, options=options)
+                try:
+                    uname, passwd = parse_user_password(rpl_user,
+                                                        options=options)
+                except FormatError:
+                    raise UtilError(USER_PASSWORD_FORMAT.format("--rpl-user"))
                 if not passwd:
                     passwd = ''
 
@@ -566,7 +573,10 @@ class Replication(object):
         result = True
 
         # Parse user and password (support login-paths)
-        r_user, r_pass = parse_user_password(rpl_user)
+        try:
+            r_user, r_pass = parse_user_password(rpl_user)
+        except FormatError:
+            raise UtilError(USER_PASSWORD_FORMAT.format("--rpl-user"))
 
         # Check to see if rpl_user is present, else create her
         if not self.create_rpl_user(r_user, r_pass)[0]:
@@ -778,7 +788,7 @@ class Master(Server):
         if options is None:
             options = {}
 
-        assert not options.get("conn_info") is None
+        assert options.get("conn_info") is not None
 
         self.options = options
         Server.__init__(self, options)
@@ -866,17 +876,21 @@ class Master(Server):
             host = clean_IPv6(host)
 
         # Create user class instance
-        user = User(self, "%s:%s@%s:%s" % (r_user, r_pass, host, port))
+        user = User(self, "{0}:{1}@{2}:{3}".format(r_user, r_pass, host, port))
         if not user.exists():
             user.create()
+            # Save current user for privilege checking
+            user.current_user = "'{0}'@'{1}'".format(r_user, host)
 
-        if not user.has_privilege("*", "*", "REPLICATION SLAVE"):
+        # Check privileges, but do not user the anonymous host
+        if not user.has_privilege("*", "*", "REPLICATION SLAVE",
+                                  globals_privs=False):
             if verbosity > 0:
                 print "# Granting replication access to replication user..."
-            query_str = "GRANT REPLICATION SLAVE ON *.* TO '%s'@'%s' " % \
-                        (r_user, host)
+            query_str = ("GRANT REPLICATION SLAVE ON *.* TO "
+                         "'{0}'@'{1}' ".format(r_user, host))
             if r_pass:
-                query_str += "IDENTIFIED BY '%s'" % r_pass
+                query_str += "IDENTIFIED BY '{0}'".format(r_pass)
 
             if ssl:
                 query_str = "{0} {1}".format(query_str, " REQUIRE SSL")
@@ -979,7 +993,7 @@ class Master(Server):
         connect_error_slaves = []
         res = self.exec_query("SHOW SLAVE HOSTS")
         verbose = self.options.get("verbose", False)
-        if not res == []:
+        if res != []:
             # Sort for conformity
             res.sort()  # pylint: disable=E1103
 
@@ -1032,7 +1046,7 @@ class Master(Server):
         """
         if self.supports_gtid == "ON":
             gtid_executed = self.exec_query("SELECT @@GLOBAL.GTID_EXECUTED")[0]
-            return 'SET @@GLOBAL.GTID_PURGED = "%s"' % gtid_executed
+            return "SET @@GLOBAL.GTID_PURGED = '{0}'".format(gtid_executed)
         else:
             return None
 
@@ -1307,7 +1321,7 @@ class Slave(Server):
         if options is None:
             options = {}
 
-        assert not options.get("conn_info") is None
+        assert options.get("conn_info") is not None
 
         self.options = options
         Server.__init__(self, options)
@@ -1572,8 +1586,8 @@ class Slave(Server):
         rpl_wild_do_table = res[0][_SLAVE_WILD_DO_TABLE]
         rpl_wild_ignore_table = res[0][_SLAVE_WILD_IGNORE_TABLE]
 
-        if (rpl_do_db or rpl_ignore_db or rpl_do_table or rpl_ignore_table
-                or rpl_wild_do_table or rpl_wild_ignore_table):
+        if (rpl_do_db or rpl_ignore_db or rpl_do_table or rpl_ignore_table or
+                rpl_wild_do_table or rpl_wild_ignore_table):
             return (rpl_do_db, rpl_ignore_db, rpl_do_table, rpl_ignore_table,
                     rpl_wild_do_table, rpl_wild_ignore_table)
         else:
@@ -1941,14 +1955,17 @@ class Slave(Server):
         if master_ssl and master_ssl not in ('0', 'OFF'):
             change_master = "{0}, MASTER_SSL = {1}".format(change_master, 1)
         if master_ssl_ca is not None:
-            change_master = ("{0}, MASTER_SSL_CA = '{1}'"
-                             ).format(change_master, master_ssl_ca)
+            change_master = (
+                "{0}, MASTER_SSL_CA = '{1}'"
+            ).format(change_master, master_ssl_ca)
         if master_ssl_cert:
-            change_master = ("{0}, MASTER_SSL_CERT = '{1}'"
-                             ).format(change_master, master_ssl_cert)
+            change_master = (
+                "{0}, MASTER_SSL_CERT = '{1}'"
+            ).format(change_master, master_ssl_cert)
         if master_ssl_key:
-            change_master = ("{0}, MASTER_SSL_KEY = '{1}'"
-                             ).format(change_master, master_ssl_key)
+            change_master = (
+                "{0}, MASTER_SSL_KEY = '{1}'"
+            ).format(change_master, master_ssl_key)
         if self.supports_gtid() == "ON":
             change_master += ", MASTER_AUTO_POSITION=1"
         elif not from_beginning:
@@ -1978,25 +1995,52 @@ class Slave(Server):
                                    "(slave status is empty)"
                                    ".".format(self.host, self.port))
             return False
-        # pylint: disable=W0633
-        m_host, m_port = self.get_master_host_port()
-        # Suppose the state is True for "Waiting for master to send event"
-        # so we can ignore it if verify_state is not given as True.
-        if verify_state:
-            state = self.get_state() == "Waiting for master to send event"
-            if not state:
-                if raise_error:
-                    raise UtilRplError("Slave '{0}:{1}' is not waiting for "
-                                       "events from master.".format(self.host,
-                                                                    self.port))
-                return False
-        if not master.is_alias(m_host) or int(m_port) != int(master.port):
+        # We must not assume there is one and only one master for a slave.
+        # Starting with 5.7.6, multi-master means a slave could have many
+        # masters, each connected via a replication channel. Thus, we must
+        # loop through the rows in the SHOW SLAVE STATUS and check every
+        # master listed. If no matches to this master is found, we can
+        # declare the slave not connected to the master otherwise, we can
+        # stop the loop when the master is found.
+        m_host = ""
+        m_port = None
+        master_found = False
+        for row in res:
+            # pylint: disable=W0633
+            m_host = row[_SLAVE_MASTER_HOST]  # get master host
+            m_port = row[_SLAVE_MASTER_PORT]  # get master port
+            # Suppose the state is True for "Waiting for master to send event"
+            # so we can ignore it if verify_state is not given as True.
+            if verify_state:
+                state = (row[_SLAVE_IO_STATE] ==
+                         "Waiting for master to send event")
+                if not state:
+                    if raise_error:
+                        raise UtilRplError("Slave '{0}:{1}' is not waiting"
+                                           " for events from master."
+                                           "".format(self.host, self.port))
+                    return False
+            # If we find a match, stop.
+            if master.is_alias(m_host) and int(m_port) == int(master.port):
+                master_found = True
+                break
+        # If no master found, report what we did find or in the case of
+        # multi-master (more than one row in SHOW SLAVE STATUS), state this
+        # master is not among the masters listed for the slave.
+        if not master_found:
             if raise_error:
-                raise UtilRplError("Slave '{0}:{1}' is configured for master "
-                                   "'{2}:{3}' and not '{4}:{5}'"
-                                   ".".format(self.host, self.port, m_host,
-                                              m_port, master.host,
-                                              master.port))
+                if len(res) > 1:
+                    raise UtilRplError("The list of masters for slave "
+                                       "'{0}:{1}' does not include master"
+                                       " '{2}:{3}'"
+                                       ".".format(self.host, self.port,
+                                                  master.host, master.port))
+                else:
+                    raise UtilRplError("Slave '{0}:{1}' is configured for "
+                                       "master '{2}:{3}' and not '{4}:{5}'"
+                                       ".".format(self.host, self.port,
+                                                  m_host, m_port,
+                                                  master.host, master.port))
             return False
         return True
 
