@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014, Oracle and/or its affiliates. All rights
+# Copyright (c) 2014, 2016 Oracle and/or its affiliates. All rights
 # reserved.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@ import sys
 import time
 import logging
 
-from mysql.utilities.exception import UtilError, UtilRplError
+from mysql.utilities.exception import FormatError, UtilError, UtilRplError
 from mysql.utilities.common.daemon import Daemon
 from mysql.utilities.common.format import print_list
 from mysql.utilities.common.ip_parser import hostname_is_ip
@@ -38,6 +38,7 @@ from mysql.utilities.common.server import connect_servers, get_server_state
 from mysql.utilities.common.replication import Replication, Master, Slave
 from mysql.utilities.common.topology import Topology
 from mysql.utilities.common.user import User
+from mysql.utilities.common.messages import USER_PASSWORD_FORMAT
 
 
 _MIN_SERVER_VERSION = (5, 6, 9)
@@ -245,12 +246,12 @@ class ReplicationMultiSource(Daemon):
             for any_priv_tuple in master_priv:
                 has_privilege = any(
                     [user_obj.has_privilege('*', '*', priv)
-                        for priv in any_priv_tuple]
+                     for priv in any_priv_tuple]
                 )
                 if not has_privilege:
                     msg = ERROR_USER_WITHOUT_PRIVILEGES.format(
                         user=master.user, host=master.host, port=master.port,
-                        operation='replication',
+                        operation='perform replication',
                         req_privileges=master_priv_str
                     )
                     self._report(msg, logging.CRITICAL, False)
@@ -271,7 +272,7 @@ class ReplicationMultiSource(Daemon):
         for any_priv_tuple in slave_priv:
             has_privilege = any(
                 [user_obj.has_privilege('*', '*', priv)
-                    for priv in any_priv_tuple]
+                 for priv in any_priv_tuple]
             )
             if not has_privilege:
                 msg = ("User '{0}' on '{1}@{2}' does not have sufficient "
@@ -294,7 +295,7 @@ class ReplicationMultiSource(Daemon):
         if host_port:
             host = host_port[0]
         if (not host or uses_ip != hostname_is_ip(slave.host) or
-           uses_ip != hostname_is_ip(host)):
+                uses_ip != hostname_is_ip(host)):
             return False
         return True
 
@@ -348,7 +349,7 @@ class ReplicationMultiSource(Daemon):
 
             # Check master for binary logging
             errors = rpl.check_master_binlog()
-            if not errors == []:
+            if errors != []:
                 raise UtilRplError(errors[0])
 
             # Setup replication
@@ -358,7 +359,10 @@ class ReplicationMultiSource(Daemon):
                 raise UtilRplError(msg)
         else:
             # Parse user and password (support login-paths)
-            (r_user, r_pass,) = parse_user_password(self.rpl_user)
+            try:
+                (r_user, r_pass,) = parse_user_password(self.rpl_user)
+            except FormatError:
+                raise UtilError(USER_PASSWORD_FORMAT.format("--rpl-user"))
 
             # Switch master and start slave
             slave.switch_master(master, r_user, r_pass)
@@ -630,6 +634,7 @@ class ReplicationMultiSource(Daemon):
         num_masters = len(self.masters_vals)
         use_rpl_setup = True
 
+        # pylint: disable=R0101
         while True:
             # Round-robin scheduling on the masters
             for idx in range(num_masters):
