@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2014, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,11 +28,12 @@ from multiprocessing.pool import ThreadPool
 
 from mysql.utilities.command.dbcompare import diff_objects, get_common_objects
 from mysql.utilities.common.database import Database
-from mysql.utilities.common.messages import ERROR_USER_WITHOUT_PRIVILEGES
+from mysql.utilities.common.gtid import (get_last_server_gtid,
+                                         gtid_set_cardinality,
+                                         gtid_set_union)
+from mysql.utilities.common.messages import (ERROR_USER_WITHOUT_PRIVILEGES,
+                                             ERROR_ANSI_QUOTES_MIX_SQL_MODE)
 from mysql.utilities.common.pattern_matching import convertSQL_LIKE2REGEXP
-from mysql.utilities.common.replication import (get_last_server_gtid,
-                                                gtid_set_cardinality,
-                                                gtid_set_union)
 from mysql.utilities.common.sql_transform import quote_with_backticks
 from mysql.utilities.common.topology import Topology
 from mysql.utilities.common.user import User
@@ -67,6 +68,41 @@ class RPLSynchronizer(object):
                                       options)
         self._slaves = self._rpl_topology.get_slaves_dict()
 
+        # Verify all the servers in the topology has or does not sql_mode set
+        # to 'ANSI_QUOTES'.
+        match_group, unmatch_group = \
+            self._rpl_topology.get_servers_with_different_sql_mode(
+                'ANSI_QUOTES'
+            )
+        # List and Raise an error if just some of the server has sql_mode set
+        # to 'ANSI_QUOTES' instead of all or none.
+        if match_group and unmatch_group:
+            sql_mode = match_group[0].select_variable("SQL_MODE")
+            if sql_mode == '':
+                sql_mode = '""'
+            sql_mode = sql_mode.replace(',', ', ')
+            print("# The SQL mode in the following servers is set to "
+                  "ANSI_QUOTES: {0}".format(sql_mode))
+            for server in match_group:
+                sql_mode = server.select_variable("SQL_MODE")
+                if sql_mode == '':
+                    sql_mode = '""'
+                sql_mode = sql_mode.replace(',', ', ')
+                print("# {0}:{1} sql_mode={2}"
+                      "".format(server.host, server.port, sql_mode))
+            print("# The SQL mode in the following servers is not set to "
+                  "ANSI_QUOTES:")
+            for server in unmatch_group:
+                sql_mode = server.select_variable("SQL_MODE")
+                if sql_mode == '':
+                    sql_mode = '""'
+                print("# {0}:{1} sql_mode={2}"
+                      "".format(server.host, server.port, sql_mode))
+
+            raise UtilError(ERROR_ANSI_QUOTES_MIX_SQL_MODE.format(
+                utility='mysqlrplsync'
+            ))
+
         # Set base server used as reference for comparisons.
         self._base_server = None
         self._base_server_key = None
@@ -149,12 +185,12 @@ class RPLSynchronizer(object):
             for any_priv_tuple in master_priv:
                 has_privilege = any(
                     [user_obj.has_privilege('*', '*', priv)
-                        for priv in any_priv_tuple]
+                     for priv in any_priv_tuple]
                 )
                 if not has_privilege:
                     raise UtilError(ERROR_USER_WITHOUT_PRIVILEGES.format(
                         user=server.user, host=server.host, port=server.port,
-                        operation='the synchronization check',
+                        operation='perform the synchronization check',
                         req_privileges=master_priv_str
                     ))
 
@@ -167,7 +203,7 @@ class RPLSynchronizer(object):
             for any_priv_tuple in slave_priv:
                 has_privilege = any(
                     [user_obj.has_privilege('*', '*', priv)
-                        for priv in any_priv_tuple]
+                     for priv in any_priv_tuple]
                 )
                 if not has_privilege:
                     raise UtilError(
@@ -277,8 +313,8 @@ class RPLSynchronizer(object):
                         print("# Slave '{0}':".format(slave_key))
                         filters_map = self._slaves_rpl_filters[slave_key]
                         for rpl_filter in filters_map:
-                            if (rpl_filter.startswith('replicate')
-                                    and filters_map[rpl_filter]):
+                            if (rpl_filter.startswith('replicate') and
+                                    filters_map[rpl_filter]):
                                 print("#   - {0}: {1}".format(
                                     rpl_filter,
                                     ', '.join(filters_map[rpl_filter])
@@ -341,26 +377,26 @@ class RPLSynchronizer(object):
                         return False
                     else:
                         return True
-                elif (rpl_filter['replicate_ignore_db']
-                        and obj_name in rpl_filter['replicate_ignore_db']):
+                elif (rpl_filter['replicate_ignore_db'] and
+                      obj_name in rpl_filter['replicate_ignore_db']):
                     return True
             else:
-                if (rpl_filter['replicate_do_table']
-                        and obj_name in rpl_filter['replicate_do_table']):
+                if (rpl_filter['replicate_do_table'] and
+                        obj_name in rpl_filter['replicate_do_table']):
                     return False
-                if (rpl_filter['replicate_ignore_table']
-                        and obj_name in rpl_filter['replicate_ignore_table']):
+                if (rpl_filter['replicate_ignore_table'] and
+                        obj_name in rpl_filter['replicate_ignore_table']):
                     return True
-                if (rpl_filter['replicate_wild_do_table']
-                        and match_regexp(obj_name,
-                                         rpl_filter['regexp_do_table'])):
+                if (rpl_filter['replicate_wild_do_table'] and
+                        match_regexp(obj_name,
+                                     rpl_filter['regexp_do_table'])):
                     return False
-                if (rpl_filter['replicate_wild_ignore_table']
-                        and match_regexp(obj_name,
-                                         rpl_filter['regexp_ignore_table'])):
+                if (rpl_filter['replicate_wild_ignore_table'] and
+                        match_regexp(obj_name,
+                                     rpl_filter['regexp_ignore_table'])):
                     return True
-                if (rpl_filter['replicate_do_table']
-                        or rpl_filter['replicate_wild_do_table']):
+                if (rpl_filter['replicate_do_table'] or
+                        rpl_filter['replicate_wild_do_table']):
                     return True
 
         # Do not filter replication for object (if no filter rule matched).
@@ -597,7 +633,7 @@ class RPLSynchronizer(object):
         # Return separated list of active and non active replication slaves.
         return active_slaves, not_active_slaves
 
-    def _compute_sync_point(self, active_slaves=None):
+    def _compute_sync_point(self, active_slaves=None, master_uuid=None):
         """Compute the GTID synchronization point.
 
         This method computes the GTID synchronization point based based on the
@@ -610,13 +646,18 @@ class RPLSynchronizer(object):
                             if the master is not available. It is assumed
                             that the list is composed by strings with the
                             format 'host@port', identifying each slave.
+        master_uuid[in]     UUID of the master server used to compute its last
+                            GTID (sync point). If not provided it is
+                            determined, but can lead to issues for servers
+                            >= 5.7.6 if specific tables are locked previously.
 
         Return a GTID set representing to synchronization point (to wait for
         slaves to catch up and stop).
         """
         if self._get_master():
             gtid_set = self._get_master().get_gtid_executed()
-            master_uuid = self._get_master().get_server_uuid()
+            master_uuid = master_uuid if master_uuid \
+                else self._get_master().get_server_uuid()
             return get_last_server_gtid(gtid_set, master_uuid)
         else:
             # Get GTID_EXECUTED on all slaves.
@@ -737,7 +778,6 @@ class RPLSynchronizer(object):
         """
         success = False
         checksum_issues = 0
-
         # If no master used then add base server (slave) to slaves to sync.
         if not self._get_master():
             slaves = slaves + [self._base_server_key]
@@ -746,6 +786,9 @@ class RPLSynchronizer(object):
         active_slaves, not_active_slaves = self._split_active_slaves(slaves)
 
         if self._get_master():
+            # Get uuid of the master server
+            master_uuid = self._get_master().get_server_uuid()
+
             # Lock the table on the master to get GTID synchronization point
             # and perform the table checksum.
             try:
@@ -753,7 +796,9 @@ class RPLSynchronizer(object):
                     "LOCK TABLES {0} READ".format(table)
                 )
 
-                last_exec_gtid = self._compute_sync_point()
+                last_exec_gtid = self._compute_sync_point(
+                    master_uuid=master_uuid
+                )
                 if self._verbosity > 2:
                     print("#   Sync point GTID: {0}".format(last_exec_gtid))
 
@@ -932,7 +977,7 @@ class RPLSynchronizer(object):
             dbs_not_in_base_srv = slave_dbs - base_server_dbs
             filtered_dbs = set(
                 [db for db in dbs_not_in_base_srv
-                    if self._is_rpl_filtered(db, slave=self._base_server_key)]
+                 if self._is_rpl_filtered(db, slave=self._base_server_key)]
             )
             dbs_not_in_base_srv -= filtered_dbs
             for db in filtered_dbs:
@@ -949,6 +994,7 @@ class RPLSynchronizer(object):
         filter_srv = None if self._get_master() else self._base_server_key
 
         # Check data consistency for each table on the base server.
+        # pylint: disable=R0101
         for db_name in base_server_dbs:
             # Skip database if filtered by defined replication rules.
             if self._is_rpl_filtered(db_name, slave=filter_srv):
@@ -974,8 +1020,8 @@ class RPLSynchronizer(object):
                         self._base_server, self._get_slave(slave_key),
                         db_name, db_name, False, options)
                     # Process tables to include/exclude from check (on slaves).
-                    if (data_to_include and db_name in data_to_include
-                            and data_to_include[db_name]):
+                    if (data_to_include and db_name in data_to_include and
+                            data_to_include[db_name]):
                         in_both = [
                             obj_row for obj_row in in_both
                             if obj_row[1][0] in data_to_include[db_name]
@@ -988,8 +1034,8 @@ class RPLSynchronizer(object):
                             obj_row for obj_row in not_in_basesrv
                             if obj_row[1][0] in data_to_include[db_name]
                         ]
-                    if (data_to_exclude and db_name in data_to_exclude
-                            and data_to_exclude[db_name]):
+                    if (data_to_exclude and db_name in data_to_exclude and
+                            data_to_exclude[db_name]):
                         in_both = [
                             obj_row for obj_row in in_both
                             if obj_row[1][0] not in data_to_exclude[db_name]
@@ -1040,20 +1086,22 @@ class RPLSynchronizer(object):
                     obj_name = db_obj[1][0]
                     # Process tables to include/exclude from check (on base
                     # server).
-                    if (data_to_include and data_to_include[db_name]
-                            and obj_name not in data_to_include[db_name]):
+                    if (data_to_include and db_name in data_to_include and
+                            data_to_include[db_name] and
+                            obj_name not in data_to_include[db_name]):
                         # Skip to the next object if not in data to include.
                         continue
-                    if (data_to_exclude and data_to_exclude[db_name]
-                            and obj_name in data_to_exclude[db_name]):
+                    if (data_to_exclude and db_name in data_to_exclude and
+                            data_to_exclude[db_name] and
+                            obj_name in data_to_exclude[db_name]):
                         # Skip to the next object if in data to exclude.
                         continue
                     checksum_task = []
                     # Check object data on all valid slaves.
                     for slave_key in slaves_to_check:
                         # Skip table if filtered by defined replication rules.
-                        if (obj_type == 'TABLE'
-                            and self._is_rpl_filtered(db_name, obj_name,
+                        if (obj_type == 'TABLE' and
+                                self._is_rpl_filtered(db_name, obj_name,
                                                       slave=slave_key)):
                             print("# [SKIP] Table '{0}' check for '{1}' - "
                                   "filtered by replication rule."
@@ -1072,8 +1120,8 @@ class RPLSynchronizer(object):
 
                         # Quote object name with backticks.
                         q_obj = '{0}.{1}'.format(
-                            quote_with_backticks(db_name),
-                            quote_with_backticks(obj_name)
+                            quote_with_backticks(db_name, db.sql_mode),
+                            quote_with_backticks(obj_name, db.sql_mode)
                         )
 
                         # Check object definition.
