@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,12 +21,13 @@ This module contains abstractions of a MySQL table and an index.
 
 import multiprocessing
 import sys
+from itertools import izip
 
 from mysql.utilities.exception import UtilError, UtilDBError
 from mysql.connector.conversion import MySQLConverter
 from mysql.utilities.common.format import print_list
-from mysql.utilities.common.database import Database
 from mysql.utilities.common.lock import Lock
+from mysql.utilities.common.pattern_matching import parse_object_name
 from mysql.utilities.common.server import Server
 from mysql.utilities.common.sql_transform import (convert_special_characters,
                                                   quote_with_backticks,
@@ -60,7 +61,7 @@ class Index(object):
         - Print index CREATE statement
     """
 
-    def __init__(self, db, index_tuple, verbose=False):
+    def __init__(self, db, index_tuple, verbose=False, sql_mode=''):
         """Constructor
 
         db[in]             Name of database
@@ -71,20 +72,22 @@ class Index(object):
 
         # Initialize and save values
         self.db = db
-        self.q_db = quote_with_backticks(db)
+        self.sql_mode = sql_mode
+        self.q_db = quote_with_backticks(db, self.sql_mode)
         self.verbose = verbose
         self.columns = []
         self.table = index_tuple[0]
-        self.q_table = quote_with_backticks(index_tuple[0])
+        self.q_table = quote_with_backticks(index_tuple[0], self.sql_mode)
         self.unique = not int(index_tuple[1])
         self.name = index_tuple[2]
-        self.q_name = quote_with_backticks(index_tuple[2])
+        self.q_name = quote_with_backticks(index_tuple[2], self.sql_mode)
         col = (index_tuple[4], index_tuple[7])
         self.columns.append(col)
         self.accept_nulls = True if index_tuple[9] else False
         self.type = index_tuple[10]
         self.compared = False                    # mark as compared for speed
         self.duplicate_of = None                 # saves duplicate index
+        # pylint: disable=R0102
         if index_tuple[7] > 0:
             self.column_subparts = True          # check subparts e.g. a(20)
         else:
@@ -107,10 +110,7 @@ class Index(object):
         if col_a[0] == col_b[0]:
             # if they both have sub_parts, compare them
             if sz_this and sz_that:
-                if sz_this <= sz_that:
-                    return True
-                else:
-                    return False
+                return (sz_this <= sz_that)
             # if this index has a sub_part and the other does
             # not, it is potentially redundant
             elif sz_this and sz_that is None:
@@ -129,37 +129,31 @@ class Index(object):
         Returns True if column list is a subset of index.
         """
 
-#        # Uniqueness counts - can't be duplicate if uniquess differs
-#        #                     except for primary keys which are always unique
-#        if index.name != "PRIMARY": # or self.name != "PRIMARY":
-#            if self.unique != index.unique:
-#                return False
         num_cols_this = len(self.columns)
         num_cols_that = len(index.columns)
-        num_cols_same = 0
+        same_size = num_cols_this == num_cols_that
         if self.type == "BTREE":
-            i = 0
-            while (i < num_cols_this) and (i < num_cols_that):
-                if num_cols_same <= i:  # Ensures first N cols are the same
-                    if self.__cmp_columns(self.columns[i], index.columns[i]):
-                        num_cols_same = num_cols_same + 1
-                    else:
-                        break
-                i = i + 1
+            indexes = izip(self.columns, index.columns)
+            for idx_pair in indexes:
+                if not self.__cmp_columns(*idx_pair):
+                    return False
+            # All index pairs are the same, so return index with smaller number
+            # of columns.
+            return num_cols_this <= num_cols_that
         else:  # HASH, RTREE, FULLTEXT
-            if (self.type == "FULLTEXT") and (num_cols_this != num_cols_that):
-                return False
-            i = 0
-            while (i < num_cols_this) and (i < num_cols_that):
-                if self.__cmp_columns(self.columns[i], index.columns[i]):
-                    num_cols_same = num_cols_same + 1
-                else:  # Ensures column lists must match
-                    num_cols_same = 0
-                    break
-                i = i + 1
-        if (num_cols_same > 0) and (num_cols_this <= num_cols_that):
-            return True
-        return False
+            if self.type != "FULLTEXT":
+                # For RTREE or HASH type indexes, an index is redundant if
+                # it has the exact same columns on the exact same order.
+                indexes = izip(self.columns, index.columns)
+                return (same_size and
+                        all((self.__cmp_columns(*idx_pair)
+                             for idx_pair in indexes)))
+            else:  # FULLTEXT index
+                # A FULLTEXT index A is redundant of FULLTEXT index B if
+                # the columns of A are a subset of B's columns, the order
+                # does not matter.
+                return all(any(self.__cmp_columns(col, icol) for
+                               icol in index.columns) for col in self.columns)
 
     def is_duplicate(self, index):
         """Compare this index with another
@@ -263,7 +257,7 @@ class Index(object):
         for col in self.columns:
             name, sub_part = (col[0], col[1])
             if backtick_quoting:
-                name = quote_with_backticks(name)
+                name = quote_with_backticks(name, self.sql_mode)
             if sub_part > 0:
                 col_str = "{0}({1})".format(name, sub_part)
             else:
@@ -334,18 +328,26 @@ class Table(object):
         self.quiet = options.get('quiet', False)
         self.server = server1
 
+        # Get sql_mode set on server
+        self.sql_mode = self.server.select_variable("SQL_MODE")
+
         # Keep table identifier considering backtick quotes
-        if is_quoted_with_backticks(name):
+        if is_quoted_with_backticks(name, self.sql_mode):
             self.q_table = name
-            self.q_db_name, self.q_tbl_name = Database.parse_object_name(name)
-            self.db_name = remove_backtick_quoting(self.q_db_name)
-            self.tbl_name = remove_backtick_quoting(self.q_tbl_name)
+            self.q_db_name, self.q_tbl_name = parse_object_name(name,
+                                                                self.sql_mode)
+            self.db_name = remove_backtick_quoting(self.q_db_name,
+                                                   self.sql_mode)
+            self.tbl_name = remove_backtick_quoting(self.q_tbl_name,
+                                                    self.sql_mode)
             self.table = ".".join([self.db_name, self.tbl_name])
         else:
             self.table = name
-            self.db_name, self.tbl_name = Database.parse_object_name(name)
-            self.q_db_name = quote_with_backticks(self.db_name)
-            self.q_tbl_name = quote_with_backticks(self.tbl_name)
+            self.db_name, self.tbl_name = parse_object_name(name,
+                                                            self.sql_mode)
+            self.q_db_name = quote_with_backticks(self.db_name, self.sql_mode)
+            self.q_tbl_name = quote_with_backticks(self.tbl_name,
+                                                   self.sql_mode)
             self.q_table = ".".join([self.q_db_name, self.q_tbl_name])
         self.obj_type = "TABLE"
         self.pri_idx = None
@@ -359,6 +361,7 @@ class Table(object):
         self.unique_not_null_indexes = None
         self.text_columns = []
         self.blob_columns = []
+        self.bit_columns = []
         self.column_format = None
         self.column_names = []
         self.column_name_type = []
@@ -396,7 +399,7 @@ class Table(object):
 
         db, table = (None, None)
         if tbl_name:
-            db, table = Database.parse_object_name(tbl_name)
+            db, table = parse_object_name(tbl_name, self.sql_mode)
         else:
             db = self.db_name
             table = self.tbl_name
@@ -429,17 +432,18 @@ class Table(object):
         col_format_values = [''] * stop
         if columns is not None:
             for col in range(0, stop):
-                if is_quoted_with_backticks(columns[col][0]):
+                if is_quoted_with_backticks(columns[col][0], self.sql_mode):
                     self.column_names.append(
-                        remove_backtick_quoting(columns[col][0]))
+                        remove_backtick_quoting(columns[col][0],
+                                                self.sql_mode))
                     self.q_column_names.append(columns[col][0])
                 else:
                     self.column_names.append(columns[col][0])
                     self.q_column_names.append(
-                        quote_with_backticks(columns[col][0]))
+                        quote_with_backticks(columns[col][0], self.sql_mode))
                 col_type = columns[col][1].lower()
-                if ('char' in col_type or 'enum' in col_type
-                        or 'set' in col_type or 'binary' in col_type):
+                if ('char' in col_type or 'enum' in col_type or
+                        'set' in col_type or 'binary' in col_type):
                     self.text_columns.append(col)
                     col_format_values[col] = "'%s'"
                 elif 'blob' in col_type or 'text'in col_type:
@@ -447,6 +451,9 @@ class Table(object):
                     col_format_values[col] = "%s"
                 elif "date" in col_type or "time" in col_type:
                     col_format_values[col] = "'%s'"
+                elif "bit" in col_type:
+                    self.bit_columns.append(col)
+                    col_format_values[col] = "%d"
                 else:
                     col_format_values[col] = "%s"
         self.column_format = "%s%s%s" % \
@@ -467,7 +474,8 @@ class Table(object):
             rows = self.server.exec_query("explain {0}".format(self.q_table))
             for row in rows:
                 self.column_names.append(row[0])
-                self.q_column_names.append(quote_with_backticks(row[0]))
+                self.q_column_names.append(quote_with_backticks(row[0],
+                                                                self.sql_mode))
 
         return self.q_column_names if quote_backticks else self.column_names
 
@@ -485,7 +493,8 @@ class Table(object):
         for row in rows:
             if quote_backticks:
                 self.column_name_type.append(
-                    [quote_with_backticks(row[0])] + list(row[1:])
+                    [quote_with_backticks(row[0], self.sql_mode)] +
+                    list(row[1:])
                 )
             else:
                 self.column_name_type.append(row)
@@ -502,7 +511,7 @@ class Table(object):
         false.
         """
         if [idx_q_name for idx_q_name in self.indexes_q_names
-           if idx_q_name == index_q_name]:
+                if idx_q_name == index_q_name]:
             return True
         return False
 
@@ -538,8 +547,8 @@ class Table(object):
                  idx.unique]
             )
             no_null_idxes.extend(
-                [idx for idx in self.fulltext_indexes if not idx.accept_nulls
-                 and idx.unique]
+                [idx for idx in self.fulltext_indexes
+                 if not idx.accept_nulls and idx.unique]
             )
             self.unique_not_null_indexes = no_null_idxes
 
@@ -569,7 +578,8 @@ class Table(object):
                     if do_commas:
                         blob_insert += ", "
                     blob_insert += "%s = " % col_name + "%s" % \
-                                   MySQLConverter().quote(row[col])
+                                   MySQLConverter().quote(
+                                       convert_special_characters(row[col]))
                     has_data = True
                     do_commas = True
             else:
@@ -581,7 +591,7 @@ class Table(object):
                 where_values.append("{0} = {1}".format(col_name, value))
         if has_data:
             return "{0} WHERE {1};".format(blob_insert,
-                   " AND ".join(where_values))
+                                           " AND ".join(where_values))
         return None
 
     def _build_insert_blob(self, row, new_db, tbl_name):
@@ -600,13 +610,20 @@ class Table(object):
         row_vals = []
         # Deal with blob, special characters and NULL values.
         for index, column in enumerate(row):
+            # pylint: disable=W0212
             if index in self.blob_columns:
-                row_vals.append(converter.quote(column))
+                row_vals.append(converter.quote(
+                    convert_special_characters(column)))
             elif index in self.text_columns:
                 if column is None:
                     row_vals.append("NULL")
                 else:
                     row_vals.append(convert_special_characters(column))
+            elif index in self.bit_columns:
+                if column is None:
+                    row_vals.append("NULL")
+                else:
+                    row_vals.append(converter._BIT_to_python(column))
             else:
                 if column is None:
                     row_vals.append("NULL")
@@ -644,8 +661,9 @@ class Table(object):
         # If all columns are blobs or there aren't any UNIQUE NOT NULL indexes
         # then rows won't be correctly copied using the update statement,
         # so we must use insert statements instead.
-        if not skip_blobs and (len(self.blob_columns) == len(self.column_names)
-                               or self.blob_columns and not unique_indexes):
+        if not skip_blobs and \
+                (len(self.blob_columns) == len(self.column_names) or
+                 self.blob_columns and not unique_indexes):
             blob_inserts.append(self._build_insert_blob(row, new_db,
                                                         self.q_tbl_name))
             is_blob_insert = True
@@ -666,11 +684,17 @@ class Table(object):
             # single quoted. For example, 'this' is it' is changed to
             # 'this\' is it'.
             for col in self.text_columns:
-                #Check if the value is not None before replacing quotes
+                # Check if the value is not None before replacing quotes
                 if values[col]:
                     # Apply escape sequences to special characters
                     values[col] = convert_special_characters(values[col])
 
+            for col in self.bit_columns:
+                if values[col] is not None:
+                    # Convert BIT to INTEGER for dump.
+                    # pylint: disable=W0212
+                    values[col] = MySQLConverter()._BIT_to_python(values[col])
+
             # Build string (add quotes to "string" like types)
             val_str = self.column_format % tuple(values)
 
@@ -733,7 +757,7 @@ class Table(object):
                 next_size = data_size + row_size + 3
                 if ((row_count >= _MAXBULK_VALUES) or
                         (next_size > (int(self.max_packet_size) - 512))):
-                   # add to buffer
+                    # add to buffer
                     data_inserts.append(insert_str)
                     row_count = 0
                 else:
@@ -833,6 +857,11 @@ class Table(object):
         dest = Server(server_options)
         dest.connect()
 
+        # Test if SQL_MODE is 'NO_BACKSLASH_ESCAPES' in the destination server
+        if dest.select_variable("SQL_MODE") == "NO_BACKSLASH_ESCAPES":
+            # Change temporarily the SQL_MODE in the destination server
+            dest.exec_query("SET @@SESSION.SQL_MODE=''")
+
         # Issue the write lock
         lock_list = [("%s.%s" % (new_db, self.q_tbl_name), 'WRITE')]
         my_lock = Lock(dest, lock_list, {'locking': 'lock-all', })
@@ -842,7 +871,6 @@ class Table(object):
 
         if self.column_format is None:
             self.get_column_metadata()
-
         data_lists = self.make_bulk_insert(rows, new_db)
         insert_data = data_lists[0]
         blob_data = data_lists[1]
@@ -937,13 +965,14 @@ class Table(object):
         new_db[in]         Rename the db to this name
         connections[in]    Number of threads(connections) to use for insert
         """
-
+        # Get sql_mode from destination
+        dest_sql_mode = destination.select_variable("SQL_MODE")
         if new_db is None:
             new_db = self.q_db_name
         else:
             # If need quote new_db identifier with backticks
-            if not is_quoted_with_backticks(new_db):
-                new_db = quote_with_backticks(new_db)
+            if not is_quoted_with_backticks(new_db, dest_sql_mode):
+                new_db = quote_with_backticks(new_db, dest_sql_mode)
 
         num_conn = int(connections)
 
@@ -952,6 +981,30 @@ class Table(object):
         else:
             # Read and copy the data
             pthreads = []
+            # Change the sql_mode if the mode is different on each server
+            # and if "ANSI_QUOTES" is set in source, this is for
+            # compatibility between the names.
+            prev_sql_mode = ''
+            if self.sql_mode != dest_sql_mode and \
+               "ANSI_QUOTES" in self.sql_mode:
+                prev_sql_mode = self.server.select_variable("SQL_MODE")
+                self.server.exec_query("SET @@SESSION.SQL_MODE=''")
+                self.sql_mode = ''
+
+                self.q_tbl_name = quote_with_backticks(
+                    self.tbl_name,
+                    self.sql_mode
+                )
+                self.q_db_name = quote_with_backticks(
+                    self.db_name,
+                    self.sql_mode
+                )
+                self.q_table = ".".join([self.q_db_name, self.q_tbl_name])
+                self.q_column_names = []
+                for column in self.column_names:
+                    self.q_column_names.append(
+                        quote_with_backticks(column, self.sql_mode)
+                    )
             for rows in self.retrieve_rows(num_conn):
                 p = self.insert_rows(rows, new_db, destination, num_conn > 1)
                 if p is not None:
@@ -962,6 +1015,25 @@ class Table(object):
                 # Wait for all threads to finish
                 for p in pthreads:
                     p.join()
+            # restoring the previous sql_mode, changed if the sql_mode in both
+            # servers is different and one is "ANSI_QUOTES"
+            if prev_sql_mode:
+                self.server.exec_query("SET @@SESSION.SQL_MODE={0}"
+                                       "".format(prev_sql_mode))
+                self.sql_mode = prev_sql_mode
+                self.q_tbl_name = quote_with_backticks(
+                    self.tbl_name,
+                    self.sql_mode
+                )
+                self.q_db_name = quote_with_backticks(
+                    self.db_name,
+                    self.sql_mode
+                )
+                self.q_table = ".".join([self.q_db_name, self.q_tbl_name])
+                for column in self.column_names:
+                    self.q_column_names.append(
+                        quote_with_backticks(column, self.sql_mode)
+                    )
 
     def retrieve_rows(self, num_conn=1):
         """Retrieve the table data in rows.
@@ -994,17 +1066,14 @@ class Table(object):
                 if row is None:
                     raise StopIteration()
                 rows.append(row)
-                #print "ROWS 1:", rows
             elif num_conn == 1:
                 rows = cur.fetchall()
-                #print "ROWS 2:", rows
                 yield rows
                 raise StopIteration()
             else:
                 rows = cur.fetchmany(segment_size)
                 if not rows:
                     raise StopIteration()
-                #print "ROWS 3:", rows
             if rows is None:
                 raise StopIteration()
             yield rows
@@ -1043,6 +1112,20 @@ class Table(object):
         Returns result set
         """
         res = self.server.exec_query("SHOW INDEXES FROM %s" % self.q_table)
+        # Clear the cardinality column
+        if res:
+            new_res = []
+            for row in res:
+                new_row = []
+                i = 0
+                for item in row:
+                    if not i == 6:
+                        new_row.append(item)
+                    else:
+                        new_row.append("0")
+                    i = i + 1
+                new_res.append(tuple(new_row))
+            res = new_res
         return res
 
     def get_tbl_foreign_keys(self):
@@ -1213,7 +1296,7 @@ class Table(object):
         for row in rows:
             if (row[2] != prev_name) or (prev_name == ""):
                 prev_name = row[2]
-                idx = Index(self.db_name, row)
+                idx = Index(self.db_name, row, sql_mode=self.sql_mode)
                 if idx.type == "BTREE":
                     self.__append(self.btree_indexes, idx)
                 elif idx.type == "HASH":
@@ -1224,7 +1307,8 @@ class Table(object):
                     self.__append(self.fulltext_indexes, idx)
             elif idx:
                 idx.add_column(row[4], row[7], row[9])
-            self.indexes_q_names.append(quote_with_backticks(row[2]))
+            self.indexes_q_names.append(quote_with_backticks(row[2],
+                                                             self.sql_mode))
         return True
 
     def check_indexes(self, show_drops=False):
@@ -1249,10 +1333,6 @@ class Table(object):
         # if there are duplicates, add them to the dupes list
         if res[0]:
             dupes.extend(res[1])
-        # We sort the fulltext index columns - easier to do it once here
-        for index in self.fulltext_indexes:
-            cols = index.columns
-            cols.sort(key=lambda cols: cols[0])
         res = self.__check_index_list(self.fulltext_indexes)
         # if there are duplicates, add them to the dupes list
         if res[0]:
