@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,19 +19,23 @@
 This module contains methods for working with mysql server tools.
 """
 
-import ctypes
 import inspect
 import os
 import re
 import sys
+import shlex
 import shutil
 import socket
 import subprocess
 import time
 
+try:
+    import ctypes
+except ImportError:
+    pass
+
 from mysql.utilities import (PYTHON_MIN_VERSION, PYTHON_MAX_VERSION,
                              CONNECTOR_MIN_VERSION)
-from mysql.utilities.common.format import print_list
 from mysql.utilities.exception import UtilError
 
 
@@ -55,7 +59,7 @@ def _add_basedir(search_paths, path_str):
 
 
 def get_tool_path(basedir, tool, fix_ext=True, required=True,
-                  defaults_paths=None, search_PATH=False):
+                  defaults_paths=None, search_PATH=False, quote=False):
     """Search for a MySQL tool and return the full path
 
     basedir[in]         The initial basedir to search (from mysql server)
@@ -71,12 +75,20 @@ def get_tool_path(basedir, tool, fix_ext=True, required=True,
                         the PATH environment variable will be used to search
                         for the tool. By default the PATH will not be searched,
                         i.e. search_PATH=False.
+    quote[in]           If True, the result path is surrounded with the OS
+                        quotes.
     Returns (string) full path to tool
     """
     if not defaults_paths:
         defaults_paths = []
     search_paths = []
-
+    if quote:
+        if os.name == "posix":
+            quote_char = "'"
+        else:
+            quote_char = '"'
+    else:
+        quote_char = ''
     if basedir:
         # Add specified basedir path to search paths
         _add_basedir(search_paths, basedir)
@@ -103,12 +115,12 @@ def get_tool_path(basedir, tool, fix_ext=True, required=True,
         if os.path.isdir(norm_path):
             toolpath = os.path.join(norm_path, tool)
             if os.path.isfile(toolpath):
-                return toolpath
+                return r"%s%s%s" % (quote_char, toolpath, quote_char)
             else:
                 if tool == "mysqld.exe":
                     toolpath = os.path.join(norm_path, "mysqld-nt.exe")
                     if os.path.isfile(toolpath):
-                        return toolpath
+                        return r"%s%s%s" % (quote_char, toolpath, quote_char)
     if required:
         raise UtilError("Cannot find location of %s." % tool)
 
@@ -190,15 +202,22 @@ def execute_script(run_cmd, filename=None, options=None, verbosity=False):
             filename = os.devnull
         f_out = open(filename, 'w')
 
-    str_opts = [str(opt) for opt in options]
-    cmd_opts = " ".join(str_opts)
-    command = " ".join([run_cmd, cmd_opts])
+    is_posix = (os.name == "posix")
+    command = shlex.split(run_cmd, posix=is_posix)
+
+    if options:
+        command.extend([str(opt) for opt in options])
 
     if verbosity:
-        print("# SCRIPT EXECUTED: {0}".format(command))
+        print("# SCRIPT EXECUTED: {0}".format(" ".join(command)))
+
+    try:
+        proc = subprocess.Popen(command, shell=False,
+                                stdout=f_out, stderr=f_out)
+    except:
+        _, err, _ = sys.exc_info()
+        raise UtilError(str(err))
 
-    proc = subprocess.Popen(command, shell=True,
-                            stdout=f_out, stderr=f_out)
     ret_val = proc.wait()
     if not verbosity:
         f_out.close()
@@ -225,6 +244,28 @@ def ping_host(host, timeout):
     return (ret_val == 0)
 
 
+def parse_mysqld_version(vers_str):
+    """ Parse the MySQL version string.
+
+    vers_str[in]     MySQL Version from client
+
+    Returns string = version string
+    """
+    pattern = r"mysqld(?:\.exe)?\s+Ver\s+(\d+\.\d+\.\S+)\s"
+    match = re.search(pattern, vers_str)
+    if not match:
+        return None
+    version = match.group(1)
+    try:
+        # get the version digits. If more than 2, we get first 3 parts
+        # pylint: disable=W0612
+        maj_ver, min_ver, dev = version.split(".", 2)
+        rel = dev.split("-", 1)
+        return (maj_ver, min_ver, rel[0])
+    except:
+        return None
+
+
 def get_mysqld_version(mysqld_path):
     """Return the version number for a mysqld executable.
 
@@ -251,15 +292,9 @@ def get_mysqld_version(mysqld_path):
 
     if line is None:
         return None
-    version = line.split(' ', 5)[3]
-    try:
-        maj_ver, min_ver, dev = version.split(".")
-        rel = dev.split("-")
-        return (maj_ver, min_ver, rel[0])
-    except:
-        return None
-
-    return None
+    # strip path for long, unusual paths that contain version number
+    fixed_str = "{0} {1}".format("mysqld", line.strip(mysqld_path))
+    return parse_mysqld_version(fixed_str)
 
 
 def show_file_statistics(file_name, wild=False, out_format="GRID"):
@@ -289,6 +324,8 @@ def show_file_statistics(file_name, wild=False, out_format="GRID"):
     else:
         rows.append(_get_file_stats(path, filename))
 
+    # Local import is needed because of Python compability issues
+    from mysql.utilities.common.format import print_list
     print_list(sys.stdout, out_format, columns, rows)
 
 
@@ -366,7 +403,7 @@ def check_python_version(min_version=PYTHON_MIN_VERSION,
             mod = inspect.getmodule(frm[0])
             mod_name = os.path.splitext(
                 os.path.basename(mod.__file__))[0]
-            name = '{0} utility'.format(mod_name)
+            name = '%s utility' % mod_name
 
         # Build the error message
         if max_version:
@@ -392,7 +429,7 @@ def check_python_version(min_version=PYTHON_MIN_VERSION,
             raise UtilError(error_msg)
 
         if print_on_fail:
-            print('ERROR: {0}'.format(error_msg))
+            print('ERROR: %s' % error_msg)
 
         if exit_on_fail:
             sys.exit(1)
@@ -571,3 +608,23 @@ def print_elapsed_time(start_time):
     stop_time = time.time()
     display_time = stop_time - start_time
     print("Time: {0:.2f} sec\n".format(display_time))
+
+
+def join_and_build_str(list_of_strings, sep=', ', last_sep='and'):
+    """Buils and returns a string from a list of elems.
+
+    list_of_strings[in]    the list of strings that will be joined into a
+                           single string.
+    sep[in]                the separator that will be used to group all strings
+                           except the last one.
+    last_sep[in]           the separator that is used in last place
+    """
+    if list_of_strings:
+        if len(list_of_strings) > 1:
+            res_str = "{0} {1} {2}".format(
+                sep.join(list_of_strings[:-1]), last_sep, list_of_strings[-1])
+        else:  # list has a single elem
+            res_str = list_of_strings[0]
+    else:  # if list_of_str is empty, return empty string
+        res_str = ""
+    return res_str
