@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,9 +23,11 @@ import re
 
 from collections import namedtuple, defaultdict
 
+from mysql.utilities.common.grants_info import filter_grants
 from mysql.utilities.exception import UtilError, UtilDBError, FormatError
 from mysql.utilities.common.ip_parser import parse_connection, clean_IPv6
 from mysql.utilities.common.messages import ERROR_USER_WITHOUT_PRIVILEGES
+from mysql.utilities.common.pattern_matching import parse_object_name
 from mysql.utilities.common.sql_transform import (is_quoted_with_backticks,
                                                   quote_with_backticks)
 
@@ -75,8 +77,12 @@ def parse_user_host(user_name):
     """Parse user, passwd, host, port from user:passwd@host
 
     user_name[in]      MySQL user string (user:passwd@host)
-    """
 
+    returns - tuple - user, passwd, host
+    """
+    # Check for anonymous user. If not, continue.
+    if user_name == "''@'%'":
+        return ('', None, '%')
     no_ticks = user_name.replace("'", "")
     try:
         conn_values = parse_connection(no_ticks)
@@ -107,7 +113,7 @@ def grant_proxy_ssl_privileges(server, user, passw, at='localhost',
     Note: Raises UtilError on any Error.
     """
 
-    grant = [
+    grant_parts = [
         "GRANT", privs,
         "ON *.*",
         "TO '{0}'@'{1}'".format(user, at),
@@ -117,7 +123,7 @@ def grant_proxy_ssl_privileges(server, user, passw, at='localhost',
     ]
 
     try:
-        server.exec_query(" ".join(grant))
+        server.exec_query(" ".join(grant_parts))
     except UtilDBError as err:
         raise UtilError("Cannot create new user {0} at {1}:{2} reason:"
                         "{3}".format(user, server.host, server.port,
@@ -203,6 +209,10 @@ class User(object):
         """
 
         self.server1 = server1
+        if server1.db_conn:
+            self.sql_mode = self.server1.select_variable("SQL_MODE")
+        else:
+            self.sql_mode = ""
         self.user, self.passwd, self.host = parse_user_host(user)
         self.verbosity = verbosity
         self.current_user = None
@@ -214,7 +224,7 @@ class User(object):
             'fetch': False
         }
 
-    def create(self, new_user=None):
+    def create(self, new_user=None, authentication=None):
         """Create the user
 
         Attempts to create the user. If the operation fails, an error is
@@ -223,19 +233,34 @@ class User(object):
         new_user[in]       MySQL user string (user@host:passwd)
                            (optional) If omitted, operation is performed
                            on the class instance user name.
+        authentication[in] Special authentication clause for non-native
+                           authentication plugins
         """
-
+        auth_str = "SELECT * FROM INFORMATION_SCHEMA.PLUGINS WHERE " \
+                   "PLUGIN_NAME = '{0}' AND PLUGIN_STATUS = 'ACTIVE';"
         query_str = "CREATE USER "
         user, passwd, host = None, None, None
         if new_user:
             user, passwd, host = parse_user_host(new_user)
-            query_str += "'%s'@'%s' " % (user, host)
+            user_host_str = "'{0}'@'{1}' ".format(user, host)
         else:
-            query_str += "'%s'@'%s' " % (self.user, self.host)
+            user_host_str = "'{0}'@'{1}' ".format(self.user, self.host)
             passwd = self.passwd
+        query_str += user_host_str
 
+        if passwd and authentication:
+            print("WARNING: using a password and an authentication plugin is "
+                  "not permited. The password will be used instead of the "
+                  "authentication plugin.")
         if passwd:
-            query_str += "IDENTIFIED BY '%s'" % (passwd)
+            query_str += "IDENTIFIED BY '{0}'".format(passwd)
+        elif authentication:
+            # need to validate authentication plugin
+            res = self.server1.exec_query(auth_str.format(authentication))
+            if (res is None) or (res == []):
+                raise UtilDBError("Plugin {0} not loaded or not active. "
+                                  "Cannot create user.".format(authentication))
+            query_str += "IDENTIFIED WITH '{0}'".format(authentication)
         if self.verbosity > 0:
             print query_str
 
@@ -288,7 +313,7 @@ class User(object):
         return (res is not None and len(res) >= 1)
 
     @staticmethod
-    def _get_grants_as_dict(grant_list, verbosity=0):
+    def _get_grants_as_dict(grant_list, verbosity=0, sql_mode=''):
         """Transforms list of grant string statements into a dictionary.
 
         grant_list[in]    List of grant strings as returned from the server
@@ -297,7 +322,7 @@ class User(object):
         """
         grant_dict = defaultdict(lambda: defaultdict(set))
         for grant in grant_list:
-            grant_tpl = User._parse_grant_statement(grant[0])
+            grant_tpl = User._parse_grant_statement(grant[0], sql_mode)
             # Ignore PROXY privilege, it is not yet supported
             if verbosity > 0:
                 if 'PROXY' in grant_tpl:
@@ -346,7 +371,8 @@ class User(object):
             # Cache user grants
             self.grant_list = grants[:]
             self.grant_dict = User._get_grants_as_dict(self.grant_list,
-                                                       self.verbosity)
+                                                       self.verbosity,
+                                                       self.sql_mode)
             # If current user is already using global host wildcard '%', there
             # is no need to run the show grants again.
             if globals_privs:
@@ -381,7 +407,72 @@ class User(object):
             else:
                 return self.grant_list
 
-    def has_privilege(self, db, obj, access, allow_skip_grant_tables=True):
+    def get_grants_for_object(self, qualified_obj_name, obj_type_str,
+                              global_privs=False):
+        """ Retrieves the list of grants that the current user has that that
+         have effect over a given object.
+
+        qualified_obj_name[in]   String with the qualified name of the object.
+        obj_type_str[in]         String with the type of the object that we are
+                                 working with, must be one of 'ROUTINE',
+                                 'TABLE' or 'DATABASE'.
+        global_privs[in]         If True, the wildcard'%' host privileges are
+                                 also taken into account
+
+
+        This method takes the MySQL privilege hierarchy into account, e.g,
+        if the qualified object is a table, it returns all the grant
+        statements for this user regarding that table, as well as the grant
+        statements for this user regarding the db where the table is at and
+        finally any global grants that the user might have.
+
+        Returns a list of strings with the grant statements.
+        """
+
+        grant_stm_lst = self.get_grants(global_privs)
+        m_objs = parse_object_name(qualified_obj_name, self.sql_mode)
+        grants = []
+        if not m_objs:
+            raise UtilError("Cannot parse the specified qualified name "
+                            "'{0}'".format(qualified_obj_name))
+        else:
+            db_name, obj_name = m_objs
+            # Quote database and object name if necessary
+            if not is_quoted_with_backticks(db_name, self.sql_mode):
+                db_name = quote_with_backticks(db_name, self.sql_mode)
+            if obj_name and obj_name != '*':
+                if not is_quoted_with_backticks(obj_name, self.sql_mode):
+                    obj_name = quote_with_backticks(obj_name, self.sql_mode)
+
+            # For each grant statement look for the ones that apply to this
+            # user and object
+            for grant_stm in grant_stm_lst:
+                grant_tpl = self._parse_grant_statement(grant_stm[0],
+                                                        self.sql_mode)
+                if grant_tpl:
+                    # Check if any of the privileges applies to this object
+                    # and if it does then check if it inherited from this
+                    # statement
+                    if filter_grants(grant_tpl.privileges, obj_type_str):
+                        # Add global grants
+                        if grant_tpl.db == '*':
+                            grants.append(grant_stm[0])
+                            continue
+                        # Add database level grants
+                        if grant_tpl.db == db_name and grant_tpl.object == '*':
+                            grants.append(grant_stm[0])
+                            continue
+                        # If it is an object, add existing object level grants
+                        # as well.
+                        if obj_name:
+                            if (grant_tpl.db == db_name and
+                                    grant_tpl.object == obj_name):
+                                grants.append(grant_stm[0])
+
+        return grants
+
+    def has_privilege(self, db, obj, access, allow_skip_grant_tables=True,
+                      globals_privs=True):
         """Check to see user has a specific access to a db.object.
 
         db[in]             Name of database
@@ -390,6 +481,8 @@ class User(object):
         allow_skip_grant_tables[in]  If True, allow silent failure for
                            cases where the server is started with
                            --skip-grant-tables. Default=True
+        globals_privs[in]  Include global privileges in clone (i.e. user@%)
+                           Default is True
 
         Returns True if user has access, False if not
         """
@@ -402,7 +495,7 @@ class User(object):
         access = access.upper()
 
         # Get grant dictionary
-        grant_dict = self.get_grants(globals_privs=True, as_dict=True)
+        grant_dict = self.get_grants(globals_privs=globals_privs, as_dict=True)
 
         # If self has all privileges for all databases, no need to check,
         # simply return True
@@ -411,11 +504,11 @@ class User(object):
             return True
 
         # Quote db and obj with backticks if necessary
-        if not is_quoted_with_backticks(db) and db != '*':
-            db = quote_with_backticks(db)
+        if not is_quoted_with_backticks(db, self.sql_mode) and db != '*':
+            db = quote_with_backticks(db, self.sql_mode)
 
-        if not is_quoted_with_backticks(obj) and obj != '*':
-            obj = quote_with_backticks(obj)
+        if not is_quoted_with_backticks(obj, self.sql_mode) and obj != '*':
+            obj = quote_with_backticks(obj, self.sql_mode)
 
         # USAGE privilege is the same as no privileges,
         # so everyone has it.
@@ -425,18 +518,18 @@ class User(object):
         # OPTION privilege.
         # Check server wide grants.
         elif (access in grant_dict['*']['*'] or
-                "ALL PRIVILEGES" in grant_dict['*']['*'] and
-                access != "GRANT OPTION"):
+              "ALL PRIVILEGES" in grant_dict['*']['*'] and
+              access != "GRANT OPTION"):
             return True
         # Check database level grants.
         elif (access in grant_dict[db]['*'] or
-                "ALL PRIVILEGES" in grant_dict[db]['*'] and
-                access != "GRANT OPTION"):
+              "ALL PRIVILEGES" in grant_dict[db]['*'] and
+              access != "GRANT OPTION"):
             return True
         # Check object level grants.
         elif (access in grant_dict[db][obj] or
-                "ALL PRIVILEGES" in grant_dict[db][obj] and
-                access != "GRANT OPTION"):
+              "ALL PRIVILEGES" in grant_dict[db][obj] and
+              access != "GRANT OPTION"):
             return True
         else:
             return False
@@ -541,6 +634,15 @@ class User(object):
         for grant_tuple in res:
             print grant_tuple[0]
 
+    def _get_authentication(self):
+        """ Return authentication string """
+        res = self.server1.exec_query("SELECT plugin FROM mysql.user "
+                                      "WHERE user='{0}' and host='{1}'"
+                                      "".format(self.user, self.host))
+        if res == [] or res[0][0] == 'mysql_native_password':
+            return None
+        return res[0][0]
+
     def clone(self, new_user, destination=None, globals_privs=False):
         """Clone the current user to the new user
 
@@ -564,9 +666,15 @@ class User(object):
             # Create an instance of the user class.
             user = User(server, new_user, self.verbosity)
             if not user.exists():
-                user.create()
+                # Get authentication plugin if different from native plugin
+                auth = self._get_authentication()
+                # Add authentication if available
+                user.create(authentication=auth)
 
-            base_user_ticks = "'" + self.user + "'@'" + self.host + "'"
+            if globals_privs and '%' in row[0]:
+                base_user_ticks = "'" + self.user + "'@'" + '%' + "'"
+            else:
+                base_user_ticks = "'" + self.user + "'@'" + self.host + "'"
             user, _, host = parse_user_host(new_user)
             new_user_ticks = "'" + user + "'@'" + host + "'"
             grant = row[0].replace(base_user_ticks, new_user_ticks, 1)
@@ -588,7 +696,7 @@ class User(object):
             res = server.exec_query(grant, self.query_options)
 
     @staticmethod
-    def _parse_grant_statement(statement):
+    def _parse_grant_statement(statement, sql_mode=''):
         """ Returns a namedtuple with the parsed GRANT information.
 
         statement[in] Grant string in the sql format returned by the server.
@@ -602,7 +710,8 @@ class User(object):
             (?:(?:(\*|`?[^']+`?)\.(\*|`?[^']+`?)) # object where grant applies
             | ('[^']*'@'[^']*')) # For proxy grants user/host
             \sTO\s([^@]+@[\S]+) # grantee
-            (?:\sIDENTIFIED\sBY\sPASSWORD(?:\s\'[^\']+\')?)? # optional pwd
+            (?:\sIDENTIFIED\sBY\sPASSWORD
+             (?:(?:\s<secret>)|(?:\s\'[^\']+\')?))? # optional pwd
             (?:\sREQUIRE\sSSL)? # optional SSL
             (\sWITH\sGRANT\sOPTION)? # optional grant option
             $ # End of grant statement
@@ -616,11 +725,11 @@ class User(object):
             # quote database name and object name with backticks
             if match.group(1).upper() != 'PROXY':
                 db = match.group(2)
-                if not is_quoted_with_backticks(db) and db != '*':
-                    db = quote_with_backticks(db)
+                if not is_quoted_with_backticks(db, sql_mode) and db != '*':
+                    db = quote_with_backticks(db, sql_mode)
                 obj = match.group(3)
-                if not is_quoted_with_backticks(obj) and obj != '*':
-                    obj = quote_with_backticks(obj)
+                if not is_quoted_with_backticks(obj, sql_mode) and obj != '*':
+                    obj = quote_with_backticks(obj, sql_mode)
             else:  # if it is not a proxy grant
                 db = obj = None
             grants = grant_tpl_factory(
