@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -191,7 +191,6 @@ class Utilities(object):
             if isinstance(res, tuple):
                 is_compat, error_msg = res
                 if not is_compat:
-                    # pylint: disable=E1103
                     self.available_utilities.remove(util_name)
                     print(WARNING_FAIL_TO_READ_OPTIONS.format(util_name))
                     print("ERROR: {0}\n".format(error_msg))
@@ -231,7 +230,7 @@ class Utilities(object):
         # filter extensions
         exts = ['.py', '.exe', '', 'pyc']
         if (parts[0] not in _EXCLUDE_UTILS and
-           (len(parts) == 1 or (len(parts) == 2 and parts[1] in exts))):
+                (len(parts) == 1 or (len(parts) == 2 and parts[1] in exts))):
             util_name = str(parts[0])
             file_ext = parts[1]
             command = "{0}{1}".format(util_name, file_ext)
@@ -279,7 +278,7 @@ class Utilities(object):
             # utilities not previously found in a previos call.
             utils = self.available_utilities
             working_utils = [util['name'] for util in self.util_list]
-            if not len(working_utils) < len(self.util_list):
+            if len(working_utils) >= len(self.util_list):
                 utils = [name for name in utils if name not in working_utils]
             if len(utils) < 1:
                 return
@@ -542,3 +541,31 @@ class Utilities(object):
             return False
 
         return util_info['usage']
+
+
+def kill_process(pid, force=False, silent=False):
+    """This function tries to kill the given subprocess.
+
+    pid [in]    Process id of the subprocess to kill.
+    force [in]  Boolean value, if False try to kill process with SIGTERM
+                (Posix only) else kill it forcefully.
+    silent[in]  If true, do no print message
+
+    Returns True if operation was successful and False otherwise.
+    """
+    res = True
+    if os.name == "posix":
+        if force:
+            os.kill(pid, subprocess.signal.SIGABRT)
+        else:
+            os.kill(pid, subprocess.signal.SIGTERM)
+    else:
+        with open(os.devnull, 'w') as f_out:
+            ret_code = subprocess.call("taskkill /F /T /PID {0}".format(pid),
+                                       shell=True, stdout=f_out, stdin=f_out)
+            if ret_code not in (0, 128):
+                res = False
+                if not silent:
+                    print("Unable to successfully kill process with PID "
+                          "{0}".format(pid))
+    return res
